# -*- coding: utf-8 -*-


def get_point(self, indices=None):
    """Return the array of the points coordinates.

    Parameters
    ----------
    self : MeshVTK
        a MeshVTK object
    indices : list
        list of the points to extract (optional)

    Returns
    -------
    points: ndarray
        Points coordinates
    """

    mesh = self.get_mesh_pv(indices=indices)

    return mesh.points



from PySide2.QtCore import QUrl, Qt
from PySide2.QtGui import QCursor, QDesktopServices, QPixmap
from PySide2.QtWidgets import QLabel


class HelpButton(QLabel):
    """A Qlabel with a ? icon, and a click event that open a link"""

    def __init__(self, *args, **kwargs):
        """Same constructor as QLineEdit + config validator"""
        self.url = "https://eomys.com/"

        # Call the QLabel constructor
        super(HelpButton, self).__init__(*args, **kwargs)

        self.setCursor(QCursor(Qt.PointingHandCursor))
        self.setPixmap(QPixmap(":/images/images/icon/help_16.png"))

    def mousePressEvent(self, event=None):
        """Open the help link in the default browser

        Parameters
        ----------
        self :
            A HelpButton object
        event :
             (Default value = None)

        Returns
        -------

        """
        QDesktopServices.openUrl(QUrl(self.url))



from ....Classes.DataKeeper import DataKeeper


def get_force_datakeeper(self, symbol_list):
    """
    Generate DataKeepers to store by default results from force module

    Parameters
    ----------
    self: VarLoad
        A VarLoad object
    symbol_list : list
        List of the existing datakeeper (to avoid duplicate)

    Returns
    -------
    dk_list: list
        list of DataKeeper
    """
    dk_list = []

    return dk_list



# -*- coding: utf-8 -*-

# File generated according to DXF_Slot.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from ...GUI.Tools.WPathSelector.WPathSelector import WPathSelector
from ...GUI.Tools.MPLCanvas import MPLCanvas2
from ...GUI.Tools.FloatEdit import FloatEdit

from pyleecan.GUI.Resources import pyleecan_rc


class Ui_DXF_Slot(object):
    def setupUi(self, DXF_Slot):
        if not DXF_Slot.objectName():
            DXF_Slot.setObjectName(u"DXF_Slot")
        DXF_Slot.resize(745, 551)
        icon = QIcon()
        icon.addFile(
            u":/images/images/icon/pyleecan_64.png", QSize(), QIcon.Normal, QIcon.Off
        )
        DXF_Slot.setWindowIcon(icon)
        self.horizontalLayout_3 = QHBoxLayout(DXF_Slot)
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.b_reset = QPushButton(DXF_Slot)
        self.b_reset.setObjectName(u"b_reset")

        self.horizontalLayout.addWidget(self.b_reset)

        self.b_cancel = QPushButton(DXF_Slot)
        self.b_cancel.setObjectName(u"b_cancel")

        self.horizontalLayout.addWidget(self.b_cancel)

        self.b_tuto = QPushButton(DXF_Slot)
        self.b_tuto.setObjectName(u"b_tuto")
        self.b_tuto.setEnabled(False)

        self.horizontalLayout.addWidget(self.b_tuto)

        self.horizontalSpacer_2 = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout.addItem(self.horizontalSpacer_2)

        self.verticalLayout_2.addLayout(self.horizontalLayout)

        self.w_viewer = MPLCanvas2(DXF_Slot)
        self.w_viewer.setObjectName(u"w_viewer")

        self.verticalLayout_2.addWidget(self.w_viewer)

        self.textBrowser = QTextBrowser(DXF_Slot)
        self.textBrowser.setObjectName(u"textBrowser")
        self.textBrowser.setMaximumSize(QSize(16777215, 200))

        self.verticalLayout_2.addWidget(self.textBrowser)

        self.horizontalLayout_3.addLayout(self.verticalLayout_2)

        self.widget = QWidget(DXF_Slot)
        self.widget.setObjectName(u"widget")
        self.widget.setMaximumSize(QSize(400, 16777215))
        self.verticalLayout = QVBoxLayout(self.widget)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.w_path_selector = WPathSelector(self.widget)
        self.w_path_selector.setObjectName(u"w_path_selector")
        sizePolicy = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.w_path_selector.sizePolicy().hasHeightForWidth()
        )
        self.w_path_selector.setSizePolicy(sizePolicy)

        self.verticalLayout.addWidget(self.w_path_selector)

        self.gridLayout = QGridLayout()
        self.gridLayout.setObjectName(u"gridLayout")
        self.in_Zs = QLabel(self.widget)
        self.in_Zs.setObjectName(u"in_Zs")

        self.gridLayout.addWidget(self.in_Zs, 0, 0, 1, 1)

        self.si_Zs = QSpinBox(self.widget)
        self.si_Zs.setObjectName(u"si_Zs")
        sizePolicy1 = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.si_Zs.sizePolicy().hasHeightForWidth())
        self.si_Zs.setSizePolicy(sizePolicy1)
        self.si_Zs.setMinimum(1)
        self.si_Zs.setMaximum(1000)
        self.si_Zs.setSingleStep(0)
        self.si_Zs.setValue(36)

        self.gridLayout.addWidget(self.si_Zs, 0, 1, 1, 1)

        self.in_wind_begin_index = QLabel(self.widget)
        self.in_wind_begin_index.setObjectName(u"in_wind_begin_index")

        self.gridLayout.addWidget(self.in_wind_begin_index, 1, 0, 1, 1)

        self.si_wind_begin_index = QSpinBox(self.widget)
        self.si_wind_begin_index.setObjectName(u"si_wind_begin_index")

        self.gridLayout.addWidget(self.si_wind_begin_index, 1, 1, 1, 1)

        self.in_wind_end_index = QLabel(self.widget)
        self.in_wind_end_index.setObjectName(u"in_wind_end_index")

        self.gridLayout.addWidget(self.in_wind_end_index, 2, 0, 1, 1)

        self.si_wind_end_index = QSpinBox(self.widget)
        self.si_wind_end_index.setObjectName(u"si_wind_end_index")

        self.gridLayout.addWidget(self.si_wind_end_index, 2, 1, 1, 1)

        self.in_type_line = QLabel(self.widget)
        self.in_type_line.setObjectName(u"in_type_line")

        self.gridLayout.addWidget(self.in_type_line, 3, 0, 1, 1)

        self.c_type_line = QComboBox(self.widget)
        self.c_type_line.addItem("")
        self.c_type_line.addItem("")
        self.c_type_line.setObjectName(u"c_type_line")

        self.gridLayout.addWidget(self.c_type_line, 3, 1, 1, 1)

        self.in_axe_angle = QLabel(self.widget)
        self.in_axe_angle.setObjectName(u"in_axe_angle")

        self.gridLayout.addWidget(self.in_axe_angle, 4, 0, 1, 1)

        self.lf_axe_angle = FloatEdit(self.widget)
        self.lf_axe_angle.setObjectName(u"lf_axe_angle")
        sizePolicy2 = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        sizePolicy2.setHorizontalStretch(0)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(
            self.lf_axe_angle.sizePolicy().hasHeightForWidth()
        )
        self.lf_axe_angle.setSizePolicy(sizePolicy2)

        self.gridLayout.addWidget(self.lf_axe_angle, 4, 1, 1, 1)

        self.verticalLayout.addLayout(self.gridLayout)

        self.horizontalLayout_5 = QHBoxLayout()
        self.horizontalLayout_5.setObjectName(u"horizontalLayout_5")
        self.horizontalLayout_5.setSizeConstraint(QLayout.SetDefaultConstraint)
        self.in_coord_center = QLabel(self.widget)
        self.in_coord_center.setObjectName(u"in_coord_center")

        self.horizontalLayout_5.addWidget(self.in_coord_center)

        self.lf_center_x = FloatEdit(self.widget)
        self.lf_center_x.setObjectName(u"lf_center_x")
        sizePolicy3 = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        sizePolicy3.setHorizontalStretch(0)
        sizePolicy3.setVerticalStretch(0)
        sizePolicy3.setHeightForWidth(self.lf_center_x.sizePolicy().hasHeightForWidth())
        self.lf_center_x.setSizePolicy(sizePolicy3)

        self.horizontalLayout_5.addWidget(self.lf_center_x)

        self.lf_center_y = FloatEdit(self.widget)
        self.lf_center_y.setObjectName(u"lf_center_y")
        self.lf_center_y.setEnabled(True)
        sizePolicy3.setHeightForWidth(self.lf_center_y.sizePolicy().hasHeightForWidth())
        self.lf_center_y.setSizePolicy(sizePolicy3)
        self.lf_center_y.setMaximumSize(QSize(137, 16777215))

        self.horizontalLayout_5.addWidget(self.lf_center_y)

        self.verticalLayout.addLayout(self.horizontalLayout_5)

        self.verticalSpacer = QSpacerItem(
            20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding
        )

        self.verticalLayout.addItem(self.verticalSpacer)

        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.in_scaling = QLabel(self.widget)
        self.in_scaling.setObjectName(u"in_scaling")

        self.horizontalLayout_4.addWidget(self.in_scaling)

        self.lf_scaling = FloatEdit(self.widget)
        self.lf_scaling.setObjectName(u"lf_scaling")
        sizePolicy3.setHeightForWidth(self.lf_scaling.sizePolicy().hasHeightForWidth())
        self.lf_scaling.setSizePolicy(sizePolicy3)

        self.horizontalLayout_4.addWidget(self.lf_scaling)

        self.verticalLayout.addLayout(self.horizontalLayout_4)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalLayout_2.setSizeConstraint(QLayout.SetFixedSize)
        self.horizontalSpacer = QSpacerItem(
            40, 20, QSizePolicy.Fixed, QSizePolicy.Minimum
        )

        self.horizontalLayout_2.addItem(self.horizontalSpacer)

        self.b_plot = QPushButton(self.widget)
        self.b_plot.setObjectName(u"b_plot")

        self.horizontalLayout_2.addWidget(self.b_plot)

        self.b_save = QPushButton(self.widget)
        self.b_save.setObjectName(u"b_save")

        self.horizontalLayout_2.addWidget(self.b_save)

        self.verticalLayout.addLayout(self.horizontalLayout_2)

        self.horizontalLayout_3.addWidget(self.widget)

        self.retranslateUi(DXF_Slot)

        QMetaObject.connectSlotsByName(DXF_Slot)

    # setupUi

    def retranslateUi(self, DXF_Slot):
        DXF_Slot.setWindowTitle(
            QCoreApplication.translate("DXF_Slot", u"Define Slot from DXF", None)
        )
        self.b_reset.setText(
            QCoreApplication.translate("DXF_Slot", u"Reset View", None)
        )
        self.b_cancel.setText(
            QCoreApplication.translate("DXF_Slot", u"Cancel Selection", None)
        )
        self.b_tuto.setText(
            QCoreApplication.translate("DXF_Slot", u"Open Tutorial", None)
        )
        self.textBrowser.setHtml(
            QCoreApplication.translate(
                "DXF_Slot",
                u'<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0//EN" "http://www.w3.org/TR/REC-html40/strict.dtd">\n'
                '<html><head><meta name="qrichtext" content="1" /><style type="text/css">\n'
                "p, li { white-space: pre-wrap; }\n"
                "</style></head><body style=\" font-family:'MS Shell Dlg 2'; font-size:7.8pt; font-weight:400; font-style:normal;\">\n"
                '<p align="justify" style=" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;"><span style=" font-size:12pt;">1) Select DXF file in [m] (or use scaling factor), spline won\'t be displayed</span></p>\n'
                '<p align="justify" style=" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;"><span style=" font-size:12pt;">2) Use mouse wheel to zoom in/out</span></p>\n'
                '<p align="justify" style=" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;"><span style=" font-size:12pt;">3) Click on lines and arc'
                "s to draw the contour of a single slot</span></p>\n"
                '<p align="justify" style=" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;"><span style=" font-size:12pt;">4) First point and last point must be on the bore radius (must match the lamination radius)</span></p>\n'
                '<p align="justify" style=" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;"><span style=" font-size:12pt;">5) The winding area is define by a part of the slot contour and a closing line:</span></p>\n'
                '<p align="justify" style=" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;"><span style=" font-size:12pt;">- The points are ordered in trigonometrical order (from bore radius to bore radius)</span></p>\n'
                '<p align="justify" style=" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;"><span style=" '
                'font-size:12pt;">- First point index is 0</span></p>\n'
                '<p align="justify" style=" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;"><span style=" font-size:12pt;">- Closing line can be either a Segment or an Arc (center 0)</span></p>\n'
                '<p align="justify" style=" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;"><span style=" font-size:12pt;">6) Plot to check and save</span></p></body></html>',
                None,
            )
        )
        self.in_Zs.setText(
            QCoreApplication.translate("DXF_Slot", u"Number of slot", None)
        )
        self.in_wind_begin_index.setText(
            QCoreApplication.translate("DXF_Slot", u"Index start of winding", None)
        )
        self.in_wind_end_index.setText(
            QCoreApplication.translate("DXF_Slot", u"Index end of winding", None)
        )
        self.in_type_line.setText(
            QCoreApplication.translate("DXF_Slot", u"Type closing line", None)
        )
        self.c_type_line.setItemText(
            0, QCoreApplication.translate("DXF_Slot", u"Segment", None)
        )
        self.c_type_line.setItemText(
            1, QCoreApplication.translate("DXF_Slot", u"Arc1", None)
        )

        self.in_axe_angle.setText(
            QCoreApplication.translate("DXF_Slot", u"Slot axe angle shift", None)
        )
        self.lf_axe_angle.setText(QCoreApplication.translate("DXF_Slot", u"0", None))
        self.in_coord_center.setText(
            QCoreApplication.translate("DXF_Slot", u"Machine center (x,y)", None)
        )
        self.lf_center_x.setText(QCoreApplication.translate("DXF_Slot", u"0", None))
        self.lf_center_y.setText(QCoreApplication.translate("DXF_Slot", u"0", None))
        self.in_scaling.setText(
            QCoreApplication.translate("DXF_Slot", u"Scaling factor", None)
        )
        self.lf_scaling.setText(QCoreApplication.translate("DXF_Slot", u"1", None))
        self.b_plot.setText(QCoreApplication.translate("DXF_Slot", u"Plot", None))
        self.b_save.setText(QCoreApplication.translate("DXF_Slot", u"Save", None))

    # retranslateUi



# -*- coding: utf-8 -*-


def get_lam_index(self, label):
    """Returns list index of the lamination with the given label
    corresponding to machine.get_lam_list(is_int_to_ext=True, key=None).
    For convenience label 'Stator' or 'Rotor' are allowed here to get respective first
    stator or rotor lamination.

    Parameters
    ----------
    self : Machine
        Machine object

    label : str
        Label of the lamination

    Returns
    -------
    index : int
        List index of the lamination, if label doesn't exists index is None
    """
    # get list of lamination labels
    labels = self.get_lam_list_label()

    # if no index is provided with label append index 0
    if len(label.split("_")) == 1:
        label += "_0"

    # check if label exists
    if label in labels:
        return labels.index(label)
    else:
        return None



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Simulation/VarSimu.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Simulation/VarSimu
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Simulation.VarSimu.run import run
except ImportError as error:
    run = error

try:
    from ..Methods.Simulation.VarSimu.set_reused_data import set_reused_data
except ImportError as error:
    set_reused_data = error

try:
    from ..Methods.Simulation.VarSimu.check_param import check_param
except ImportError as error:
    check_param = error

try:
    from ..Methods.Simulation.VarSimu.generate_simulation_list import (
        generate_simulation_list,
    )
except ImportError as error:
    generate_simulation_list = error


from ._check import InitUnKnowClassError
from .DataKeeper import DataKeeper
from .Post import Post


class VarSimu(FrozenClass):
    """Abstract class for the multi-simulation"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Simulation.VarSimu.run
    if isinstance(run, ImportError):
        run = property(
            fget=lambda x: raise_(
                ImportError("Can't use VarSimu method run: " + str(run))
            )
        )
    else:
        run = run
    # cf Methods.Simulation.VarSimu.set_reused_data
    if isinstance(set_reused_data, ImportError):
        set_reused_data = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use VarSimu method set_reused_data: " + str(set_reused_data)
                )
            )
        )
    else:
        set_reused_data = set_reused_data
    # cf Methods.Simulation.VarSimu.check_param
    if isinstance(check_param, ImportError):
        check_param = property(
            fget=lambda x: raise_(
                ImportError("Can't use VarSimu method check_param: " + str(check_param))
            )
        )
    else:
        check_param = check_param
    # cf Methods.Simulation.VarSimu.generate_simulation_list
    if isinstance(generate_simulation_list, ImportError):
        generate_simulation_list = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use VarSimu method generate_simulation_list: "
                    + str(generate_simulation_list)
                )
            )
        )
    else:
        generate_simulation_list = generate_simulation_list
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        name="",
        desc="",
        datakeeper_list=-1,
        is_keep_all_output=False,
        stop_if_error=False,
        var_simu=None,
        nb_simu=0,
        is_reuse_femm_file=True,
        postproc_list=-1,
        pre_keeper_postproc_list=None,
        post_keeper_postproc_list=None,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "name" in list(init_dict.keys()):
                name = init_dict["name"]
            if "desc" in list(init_dict.keys()):
                desc = init_dict["desc"]
            if "datakeeper_list" in list(init_dict.keys()):
                datakeeper_list = init_dict["datakeeper_list"]
            if "is_keep_all_output" in list(init_dict.keys()):
                is_keep_all_output = init_dict["is_keep_all_output"]
            if "stop_if_error" in list(init_dict.keys()):
                stop_if_error = init_dict["stop_if_error"]
            if "var_simu" in list(init_dict.keys()):
                var_simu = init_dict["var_simu"]
            if "nb_simu" in list(init_dict.keys()):
                nb_simu = init_dict["nb_simu"]
            if "is_reuse_femm_file" in list(init_dict.keys()):
                is_reuse_femm_file = init_dict["is_reuse_femm_file"]
            if "postproc_list" in list(init_dict.keys()):
                postproc_list = init_dict["postproc_list"]
            if "pre_keeper_postproc_list" in list(init_dict.keys()):
                pre_keeper_postproc_list = init_dict["pre_keeper_postproc_list"]
            if "post_keeper_postproc_list" in list(init_dict.keys()):
                post_keeper_postproc_list = init_dict["post_keeper_postproc_list"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.name = name
        self.desc = desc
        self.datakeeper_list = datakeeper_list
        self.is_keep_all_output = is_keep_all_output
        self.stop_if_error = stop_if_error
        self.var_simu = var_simu
        self.nb_simu = nb_simu
        self.is_reuse_femm_file = is_reuse_femm_file
        self.postproc_list = postproc_list
        self.pre_keeper_postproc_list = pre_keeper_postproc_list
        self.post_keeper_postproc_list = post_keeper_postproc_list

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        VarSimu_str = ""
        if self.parent is None:
            VarSimu_str += "parent = None " + linesep
        else:
            VarSimu_str += "parent = " + str(type(self.parent)) + " object" + linesep
        VarSimu_str += 'name = "' + str(self.name) + '"' + linesep
        VarSimu_str += 'desc = "' + str(self.desc) + '"' + linesep
        if len(self.datakeeper_list) == 0:
            VarSimu_str += "datakeeper_list = []" + linesep
        for ii in range(len(self.datakeeper_list)):
            tmp = (
                self.datakeeper_list[ii].__str__().replace(linesep, linesep + "\t")
                + linesep
            )
            VarSimu_str += (
                "datakeeper_list[" + str(ii) + "] =" + tmp + linesep + linesep
            )
        VarSimu_str += "is_keep_all_output = " + str(self.is_keep_all_output) + linesep
        VarSimu_str += "stop_if_error = " + str(self.stop_if_error) + linesep
        if self.var_simu is not None:
            tmp = self.var_simu.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            VarSimu_str += "var_simu = " + tmp
        else:
            VarSimu_str += "var_simu = None" + linesep + linesep
        VarSimu_str += "nb_simu = " + str(self.nb_simu) + linesep
        VarSimu_str += "is_reuse_femm_file = " + str(self.is_reuse_femm_file) + linesep
        if len(self.postproc_list) == 0:
            VarSimu_str += "postproc_list = []" + linesep
        for ii in range(len(self.postproc_list)):
            tmp = (
                self.postproc_list[ii].__str__().replace(linesep, linesep + "\t")
                + linesep
            )
            VarSimu_str += "postproc_list[" + str(ii) + "] =" + tmp + linesep + linesep
        if len(self.pre_keeper_postproc_list) == 0:
            VarSimu_str += "pre_keeper_postproc_list = []" + linesep
        for ii in range(len(self.pre_keeper_postproc_list)):
            tmp = (
                self.pre_keeper_postproc_list[ii]
                .__str__()
                .replace(linesep, linesep + "\t")
                + linesep
            )
            VarSimu_str += (
                "pre_keeper_postproc_list[" + str(ii) + "] =" + tmp + linesep + linesep
            )
        if len(self.post_keeper_postproc_list) == 0:
            VarSimu_str += "post_keeper_postproc_list = []" + linesep
        for ii in range(len(self.post_keeper_postproc_list)):
            tmp = (
                self.post_keeper_postproc_list[ii]
                .__str__()
                .replace(linesep, linesep + "\t")
                + linesep
            )
            VarSimu_str += (
                "post_keeper_postproc_list[" + str(ii) + "] =" + tmp + linesep + linesep
            )
        return VarSimu_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if other.name != self.name:
            return False
        if other.desc != self.desc:
            return False
        if other.datakeeper_list != self.datakeeper_list:
            return False
        if other.is_keep_all_output != self.is_keep_all_output:
            return False
        if other.stop_if_error != self.stop_if_error:
            return False
        if other.var_simu != self.var_simu:
            return False
        if other.nb_simu != self.nb_simu:
            return False
        if other.is_reuse_femm_file != self.is_reuse_femm_file:
            return False
        if other.postproc_list != self.postproc_list:
            return False
        if other.pre_keeper_postproc_list != self.pre_keeper_postproc_list:
            return False
        if other.post_keeper_postproc_list != self.post_keeper_postproc_list:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()
        if other._name != self._name:
            diff_list.append(name + ".name")
        if other._desc != self._desc:
            diff_list.append(name + ".desc")
        if (other.datakeeper_list is None and self.datakeeper_list is not None) or (
            other.datakeeper_list is not None and self.datakeeper_list is None
        ):
            diff_list.append(name + ".datakeeper_list None mismatch")
        elif self.datakeeper_list is None:
            pass
        elif len(other.datakeeper_list) != len(self.datakeeper_list):
            diff_list.append("len(" + name + ".datakeeper_list)")
        else:
            for ii in range(len(other.datakeeper_list)):
                diff_list.extend(
                    self.datakeeper_list[ii].compare(
                        other.datakeeper_list[ii],
                        name=name + ".datakeeper_list[" + str(ii) + "]",
                    )
                )
        if other._is_keep_all_output != self._is_keep_all_output:
            diff_list.append(name + ".is_keep_all_output")
        if other._stop_if_error != self._stop_if_error:
            diff_list.append(name + ".stop_if_error")
        if (other.var_simu is None and self.var_simu is not None) or (
            other.var_simu is not None and self.var_simu is None
        ):
            diff_list.append(name + ".var_simu None mismatch")
        elif self.var_simu is not None:
            diff_list.extend(
                self.var_simu.compare(other.var_simu, name=name + ".var_simu")
            )
        if other._nb_simu != self._nb_simu:
            diff_list.append(name + ".nb_simu")
        if other._is_reuse_femm_file != self._is_reuse_femm_file:
            diff_list.append(name + ".is_reuse_femm_file")
        if (other.postproc_list is None and self.postproc_list is not None) or (
            other.postproc_list is not None and self.postproc_list is None
        ):
            diff_list.append(name + ".postproc_list None mismatch")
        elif self.postproc_list is None:
            pass
        elif len(other.postproc_list) != len(self.postproc_list):
            diff_list.append("len(" + name + ".postproc_list)")
        else:
            for ii in range(len(other.postproc_list)):
                diff_list.extend(
                    self.postproc_list[ii].compare(
                        other.postproc_list[ii],
                        name=name + ".postproc_list[" + str(ii) + "]",
                    )
                )
        if (
            other.pre_keeper_postproc_list is None
            and self.pre_keeper_postproc_list is not None
        ) or (
            other.pre_keeper_postproc_list is not None
            and self.pre_keeper_postproc_list is None
        ):
            diff_list.append(name + ".pre_keeper_postproc_list None mismatch")
        elif self.pre_keeper_postproc_list is None:
            pass
        elif len(other.pre_keeper_postproc_list) != len(self.pre_keeper_postproc_list):
            diff_list.append("len(" + name + ".pre_keeper_postproc_list)")
        else:
            for ii in range(len(other.pre_keeper_postproc_list)):
                diff_list.extend(
                    self.pre_keeper_postproc_list[ii].compare(
                        other.pre_keeper_postproc_list[ii],
                        name=name + ".pre_keeper_postproc_list[" + str(ii) + "]",
                    )
                )
        if (
            other.post_keeper_postproc_list is None
            and self.post_keeper_postproc_list is not None
        ) or (
            other.post_keeper_postproc_list is not None
            and self.post_keeper_postproc_list is None
        ):
            diff_list.append(name + ".post_keeper_postproc_list None mismatch")
        elif self.post_keeper_postproc_list is None:
            pass
        elif len(other.post_keeper_postproc_list) != len(
            self.post_keeper_postproc_list
        ):
            diff_list.append("len(" + name + ".post_keeper_postproc_list)")
        else:
            for ii in range(len(other.post_keeper_postproc_list)):
                diff_list.extend(
                    self.post_keeper_postproc_list[ii].compare(
                        other.post_keeper_postproc_list[ii],
                        name=name + ".post_keeper_postproc_list[" + str(ii) + "]",
                    )
                )
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object
        S += getsizeof(self.name)
        S += getsizeof(self.desc)
        if self.datakeeper_list is not None:
            for value in self.datakeeper_list:
                S += getsizeof(value)
        S += getsizeof(self.is_keep_all_output)
        S += getsizeof(self.stop_if_error)
        S += getsizeof(self.var_simu)
        S += getsizeof(self.nb_simu)
        S += getsizeof(self.is_reuse_femm_file)
        if self.postproc_list is not None:
            for value in self.postproc_list:
                S += getsizeof(value)
        if self.pre_keeper_postproc_list is not None:
            for value in self.pre_keeper_postproc_list:
                S += getsizeof(value)
        if self.post_keeper_postproc_list is not None:
            for value in self.post_keeper_postproc_list:
                S += getsizeof(value)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        VarSimu_dict = dict()
        VarSimu_dict["name"] = self.name
        VarSimu_dict["desc"] = self.desc
        if self.datakeeper_list is None:
            VarSimu_dict["datakeeper_list"] = None
        else:
            VarSimu_dict["datakeeper_list"] = list()
            for obj in self.datakeeper_list:
                if obj is not None:
                    VarSimu_dict["datakeeper_list"].append(obj.as_dict(**kwargs))
                else:
                    VarSimu_dict["datakeeper_list"].append(None)
        VarSimu_dict["is_keep_all_output"] = self.is_keep_all_output
        VarSimu_dict["stop_if_error"] = self.stop_if_error
        if self.var_simu is None:
            VarSimu_dict["var_simu"] = None
        else:
            VarSimu_dict["var_simu"] = self.var_simu.as_dict(**kwargs)
        VarSimu_dict["nb_simu"] = self.nb_simu
        VarSimu_dict["is_reuse_femm_file"] = self.is_reuse_femm_file
        if self.postproc_list is None:
            VarSimu_dict["postproc_list"] = None
        else:
            VarSimu_dict["postproc_list"] = list()
            for obj in self.postproc_list:
                if obj is not None:
                    VarSimu_dict["postproc_list"].append(obj.as_dict(**kwargs))
                else:
                    VarSimu_dict["postproc_list"].append(None)
        if self.pre_keeper_postproc_list is None:
            VarSimu_dict["pre_keeper_postproc_list"] = None
        else:
            VarSimu_dict["pre_keeper_postproc_list"] = list()
            for obj in self.pre_keeper_postproc_list:
                if obj is not None:
                    VarSimu_dict["pre_keeper_postproc_list"].append(
                        obj.as_dict(**kwargs)
                    )
                else:
                    VarSimu_dict["pre_keeper_postproc_list"].append(None)
        if self.post_keeper_postproc_list is None:
            VarSimu_dict["post_keeper_postproc_list"] = None
        else:
            VarSimu_dict["post_keeper_postproc_list"] = list()
            for obj in self.post_keeper_postproc_list:
                if obj is not None:
                    VarSimu_dict["post_keeper_postproc_list"].append(
                        obj.as_dict(**kwargs)
                    )
                else:
                    VarSimu_dict["post_keeper_postproc_list"].append(None)
        # The class name is added to the dict for deserialisation purpose
        VarSimu_dict["__class__"] = "VarSimu"
        return VarSimu_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.name = None
        self.desc = None
        self.datakeeper_list = None
        self.is_keep_all_output = None
        self.stop_if_error = None
        if self.var_simu is not None:
            self.var_simu._set_None()
        self.nb_simu = None
        self.is_reuse_femm_file = None
        self.postproc_list = None
        self.pre_keeper_postproc_list = None
        self.post_keeper_postproc_list = None

    def _get_name(self):
        """getter of name"""
        return self._name

    def _set_name(self, value):
        """setter of name"""
        check_var("name", value, "str")
        self._name = value

    name = property(
        fget=_get_name,
        fset=_set_name,
        doc=u"""Name of the multi-simulation

        :Type: str
        """,
    )

    def _get_desc(self):
        """getter of desc"""
        return self._desc

    def _set_desc(self, value):
        """setter of desc"""
        check_var("desc", value, "str")
        self._desc = value

    desc = property(
        fget=_get_desc,
        fset=_set_desc,
        doc=u"""Multi-simulation description

        :Type: str
        """,
    )

    def _get_datakeeper_list(self):
        """getter of datakeeper_list"""
        if self._datakeeper_list is not None:
            for obj in self._datakeeper_list:
                if obj is not None:
                    obj.parent = self
        return self._datakeeper_list

    def _set_datakeeper_list(self, value):
        """setter of datakeeper_list"""
        if type(value) is list:
            for ii, obj in enumerate(value):
                if type(obj) is dict:
                    class_obj = import_class(
                        "pyleecan.Classes", obj.get("__class__"), "datakeeper_list"
                    )
                    value[ii] = class_obj(init_dict=obj)
                if value[ii] is not None:
                    value[ii].parent = self
        if value == -1:
            value = list()
        check_var("datakeeper_list", value, "[DataKeeper]")
        self._datakeeper_list = value

    datakeeper_list = property(
        fget=_get_datakeeper_list,
        fset=_set_datakeeper_list,
        doc=u"""List containing DataKeepers to extract VarSimu results 

        :Type: [DataKeeper]
        """,
    )

    def _get_is_keep_all_output(self):
        """getter of is_keep_all_output"""
        return self._is_keep_all_output

    def _set_is_keep_all_output(self, value):
        """setter of is_keep_all_output"""
        check_var("is_keep_all_output", value, "bool")
        self._is_keep_all_output = value

    is_keep_all_output = property(
        fget=_get_is_keep_all_output,
        fset=_set_is_keep_all_output,
        doc=u"""True to store every output in a list

        :Type: bool
        """,
    )

    def _get_stop_if_error(self):
        """getter of stop_if_error"""
        return self._stop_if_error

    def _set_stop_if_error(self, value):
        """setter of stop_if_error"""
        check_var("stop_if_error", value, "bool")
        self._stop_if_error = value

    stop_if_error = property(
        fget=_get_stop_if_error,
        fset=_set_stop_if_error,
        doc=u"""Stop the multi-simulation if a simulation fails 

        :Type: bool
        """,
    )

    def _get_var_simu(self):
        """getter of var_simu"""
        return self._var_simu

    def _set_var_simu(self, value):
        """setter of var_simu"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "var_simu"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = VarSimu()
        check_var("var_simu", value, "VarSimu")
        self._var_simu = value

        if self._var_simu is not None:
            self._var_simu.parent = self

    var_simu = property(
        fget=_get_var_simu,
        fset=_set_var_simu,
        doc=u"""Multi-simulation of a Multi-simulation definition

        :Type: VarSimu
        """,
    )

    def _get_nb_simu(self):
        """getter of nb_simu"""
        return self._nb_simu

    def _set_nb_simu(self, value):
        """setter of nb_simu"""
        check_var("nb_simu", value, "int")
        self._nb_simu = value

    nb_simu = property(
        fget=_get_nb_simu,
        fset=_set_nb_simu,
        doc=u"""Number of simulations

        :Type: int
        """,
    )

    def _get_is_reuse_femm_file(self):
        """getter of is_reuse_femm_file"""
        return self._is_reuse_femm_file

    def _set_is_reuse_femm_file(self, value):
        """setter of is_reuse_femm_file"""
        check_var("is_reuse_femm_file", value, "bool")
        self._is_reuse_femm_file = value

    is_reuse_femm_file = property(
        fget=_get_is_reuse_femm_file,
        fset=_set_is_reuse_femm_file,
        doc=u"""True to reuse the femm file for each simulation (draw the machine only once, MagFEMM only)

        :Type: bool
        """,
    )

    def _get_postproc_list(self):
        """getter of postproc_list"""
        if self._postproc_list is not None:
            for obj in self._postproc_list:
                if obj is not None:
                    obj.parent = self
        return self._postproc_list

    def _set_postproc_list(self, value):
        """setter of postproc_list"""
        if type(value) is list:
            for ii, obj in enumerate(value):
                if type(obj) is dict:
                    class_obj = import_class(
                        "pyleecan.Classes", obj.get("__class__"), "postproc_list"
                    )
                    value[ii] = class_obj(init_dict=obj)
                if value[ii] is not None:
                    value[ii].parent = self
        if value == -1:
            value = list()
        check_var("postproc_list", value, "[Post]")
        self._postproc_list = value

    postproc_list = property(
        fget=_get_postproc_list,
        fset=_set_postproc_list,
        doc=u"""List of post-processing to run on XOutput after the multisimulation

        :Type: [Post]
        """,
    )

    def _get_pre_keeper_postproc_list(self):
        """getter of pre_keeper_postproc_list"""
        if self._pre_keeper_postproc_list is not None:
            for obj in self._pre_keeper_postproc_list:
                if obj is not None:
                    obj.parent = self
        return self._pre_keeper_postproc_list

    def _set_pre_keeper_postproc_list(self, value):
        """setter of pre_keeper_postproc_list"""
        if type(value) is list:
            for ii, obj in enumerate(value):
                if type(obj) is dict:
                    class_obj = import_class(
                        "pyleecan.Classes",
                        obj.get("__class__"),
                        "pre_keeper_postproc_list",
                    )
                    value[ii] = class_obj(init_dict=obj)
                if value[ii] is not None:
                    value[ii].parent = self
        if value == -1:
            value = list()
        check_var("pre_keeper_postproc_list", value, "[Post]")
        self._pre_keeper_postproc_list = value

    pre_keeper_postproc_list = property(
        fget=_get_pre_keeper_postproc_list,
        fset=_set_pre_keeper_postproc_list,
        doc=u"""If not None, replace the reference simulation postproc_list in each generated simulation (run before datakeeper)

        :Type: [Post]
        """,
    )

    def _get_post_keeper_postproc_list(self):
        """getter of post_keeper_postproc_list"""
        if self._post_keeper_postproc_list is not None:
            for obj in self._post_keeper_postproc_list:
                if obj is not None:
                    obj.parent = self
        return self._post_keeper_postproc_list

    def _set_post_keeper_postproc_list(self, value):
        """setter of post_keeper_postproc_list"""
        if type(value) is list:
            for ii, obj in enumerate(value):
                if type(obj) is dict:
                    class_obj = import_class(
                        "pyleecan.Classes",
                        obj.get("__class__"),
                        "post_keeper_postproc_list",
                    )
                    value[ii] = class_obj(init_dict=obj)
                if value[ii] is not None:
                    value[ii].parent = self
        if value == -1:
            value = list()
        check_var("post_keeper_postproc_list", value, "[Post]")
        self._post_keeper_postproc_list = value

    post_keeper_postproc_list = property(
        fget=_get_post_keeper_postproc_list,
        fset=_set_post_keeper_postproc_list,
        doc=u"""List of post-processing to run on output after each simulation (except reference one) after the datakeeper.

        :Type: [Post]
        """,
    )



# -*- coding: utf-8 -*-

from numpy import sin


def comp_width(self):
    """Compute the width of the Hole (cf schematics)

    Parameters
    ----------
    self : HoleM51
        A HoleM51 object

    Returns
    -------
    Whole: float
        Width of the Hole (cf schematics) [m]

    """

    Rext = self.get_Rext()
    return 2 * sin(self.W1 / 2) * (Rext - self.H1)



from numpy import pi


def comp_surface(self):
    """Compute the Slot total surface (by analytical computation).
    Caution, the bottom of the Slot is an Arc

    Parameters
    ----------
    self : SlotW16
        A SlotW16 object

    Returns
    -------
    S: float
        Slot total surface [m**2]

    """
    Rbo = self.get_Rbo()

    Swind = self.comp_surface_active()
    S0 = (pi * Rbo ** 2 - pi * (Rbo - self.H0) ** 2) * self.W0 / (2 * pi)

    return Swind + S0



from numpy import arcsin, exp, pi


def _comp_point_coordinate(self):
    """Compute the point coordinates needed to plot the Slot.

    Parameters
    ----------
    self : SlotW25
        A SlotW25 object

    Returns
    -------
    point_dict: dict
        A dict of the slot point coordinates
    """
    Rbo = self.get_Rbo()

    # We compute the point on the tooth, then rotation to get the slot
    # Zxt is the equivalent of point Zx in tooth ref

    # alpha_0 is the angle to rotate Z0 so ||Z1,Z8|| = W4
    alpha_0 = float(arcsin(self.W4 / (2 * Rbo)))

    # comp point coordinate (in complex)
    Z0 = Rbo * exp(1j * 0)
    Z1t = Z0 * exp(1j * alpha_0)

    if self.is_outwards():
        Z2t = Z1t + self.H1
        # alpha_1 is the angle of Z3t so ||Z3t,Z6t|| = W3
        alpha_1 = float(arcsin(self.W3 / (2 * abs(Z2t))))
        Z3t = abs(Z2t) * exp(1j * alpha_1)
        Z4t = Z3t + self.H2
    else:  # inward slot
        Z2t = Z1t - self.H1
        # alpha_1 is the angle of Z3t so ||Z3t,Z6t|| = W3
        alpha_1 = float(arcsin(self.W3 / (2 * abs(Z2t))))
        Z3t = abs(Z2t) * exp(1j * alpha_1)
        Z4t = Z3t - self.H2

    point_dict = dict()
    # Rotation to get the slot
    hssp = pi / self.Zs
    point_dict["Z1"] = Z1t * exp(-1j * hssp)
    point_dict["Z2"] = Z2t * exp(-1j * hssp)
    point_dict["Z3"] = Z3t * exp(-1j * hssp)
    point_dict["Z4"] = Z4t * exp(-1j * hssp)
    # symetry
    point_dict["Z5"] = point_dict["Z4"].conjugate()
    point_dict["Z6"] = point_dict["Z3"].conjugate()
    point_dict["Z7"] = point_dict["Z2"].conjugate()
    point_dict["Z8"] = point_dict["Z1"].conjugate()

    return point_dict



# -*- coding: utf-8 -*-

from numpy import pi

from ....Methods.Slot.Slot import SlotCheckError
from ....Methods.Slot.HoleM54 import *


def check(self):
    """Check that the HoleM54 object is correct

    Parameters
    ----------
    self : HoleM54
        A HoleM54 object

    Returns
    -------
    None

    Raises
    -------
    H54_W0CheckError
        You must have W0 < 2*pi/Zh
    H54_R1CheckError
        You must have H0 < R1
    """

    # Check that everything is set
    if self.W0 is None:
        raise S54_NoneError("You must set W0 !")
    elif self.R1 is None:
        raise S54_NoneError("You must set R1 !")
    elif self.H0 is None:
        raise S54_NoneError("You must set H0 !")
    elif self.H1 is None:
        raise S54_NoneError("You must set H1 !")

    if 2 * pi / self.Zh <= self.W0:
        raise H54_W0CheckError("You must have W0 < 2*pi/Zh")

    if self.R1 <= self.H0:
        raise H54_R1CheckError("You must have H0 < R1")



# -*- coding: utf-8 -*-

import numpy as np


def jacobian(self, point, vertices):
    """Compute jacobian, jacobian determinant and jacobian derivatives for linear triangle.

    Parameters
    ----------
    :param self : a RefElement object

    Returns
    -------

    """

    grad_func = self.grad_shape_function(point)
    jacob = np.dot(grad_func, vertices)
    det_jacob = (
        np.sqrt(
            (vertices[1, 0] - vertices[0, 0]) ** 2
            + (vertices[1, 1] - vertices[0, 1]) ** 2
        )
        / 2
    )

    return jacob, det_jacob



# -*- coding: utf-8 -*-

from numpy import pi
from PySide2.QtCore import Signal
from PySide2.QtWidgets import QMessageBox, QWidget

from .....Classes.LamSlotWind import LamSlotWind
from .....Classes.Slot import Slot
from .....Classes.SlotW10 import SlotW10
from .....Classes.Slot import Slot
from .....GUI.Dialog.DMachineSetup.SWSlot.Gen_SWSlot import Gen_SWSlot
from .....GUI.Dialog.DMachineSetup.SWSlot.PWSlotUD.PWSlotUD import PWSlotUD
from .....GUI.Dialog.DMachineSetup.SWSlot.PWSlot10.PWSlot10 import PWSlot10
from .....GUI.Dialog.DMachineSetup.SWSlot.PWSlot11.PWSlot11 import PWSlot11
from .....GUI.Dialog.DMachineSetup.SWSlot.PWSlot12.PWSlot12 import PWSlot12
from .....GUI.Dialog.DMachineSetup.SWSlot.PWSlot13.PWSlot13 import PWSlot13
from .....GUI.Dialog.DMachineSetup.SWSlot.PWSlot14.PWSlot14 import PWSlot14
from .....GUI.Dialog.DMachineSetup.SWSlot.PWSlot15.PWSlot15 import PWSlot15
from .....GUI.Dialog.DMachineSetup.SWSlot.PWSlot16.PWSlot16 import PWSlot16
from .....GUI.Dialog.DMachineSetup.SWSlot.PWSlot21.PWSlot21 import PWSlot21
from .....GUI.Dialog.DMachineSetup.SWSlot.PWSlot22.PWSlot22 import PWSlot22
from .....GUI.Dialog.DMachineSetup.SWSlot.PWSlot23.PWSlot23 import PWSlot23
from .....GUI.Dialog.DMachineSetup.SWSlot.PWSlot24.PWSlot24 import PWSlot24
from .....GUI.Dialog.DMachineSetup.SWSlot.PWSlot25.PWSlot25 import PWSlot25
from .....GUI.Dialog.DMachineSetup.SWSlot.PWSlot26.PWSlot26 import PWSlot26
from .....GUI.Dialog.DMachineSetup.SWSlot.PWSlot27.PWSlot27 import PWSlot27
from .....GUI.Dialog.DMachineSetup.SWSlot.PWSlot28.PWSlot28 import PWSlot28
from .....GUI.Dialog.DMachineSetup.SWSlot.PWSlot29.PWSlot29 import PWSlot29

# List to convert index of combobox to slot type
WIDGET_LIST = [
    PWSlot10,
    PWSlot11,
    PWSlot12,
    PWSlot13,
    PWSlot14,
    PWSlot15,
    PWSlot16,
    PWSlot21,
    PWSlot22,
    PWSlot23,
    PWSlot24,
    PWSlot25,
    PWSlot26,
    PWSlot27,
    PWSlot28,
    PWSlot29,
    PWSlotUD,
]
INIT_INDEX = [wid.slot_type for wid in WIDGET_LIST]
SLOT_NAME = [wid.slot_name for wid in WIDGET_LIST]


class SWSlot(Gen_SWSlot, QWidget):
    """Step to set the slot with winding"""

    # Signal to DMachineSetup to know that the save popup is needed
    saveNeeded = Signal()
    # Information for DMachineSetup nav
    step_name = "Slot"

    def __init__(self, machine, matlib, is_stator=False):
        """Initialize the GUI according to machine

        Parameters
        ----------
        self : SWSlot
            A SWSlot widget
        machine : Machine
            current machine to edit
        matlib : MatLib
            Material Library
        is_stator : bool
            To adapt the GUI to set either the stator or the rotor
        """

        # Build the interface according to the .ui file
        QWidget.__init__(self)
        self.setupUi(self)

        # Saving arguments
        self.machine = machine
        self.matlib = matlib
        self.is_stator = is_stator

        self.b_help.hide()

        # Fill the combobox with the available slot
        self.c_slot_type.clear()
        for slot in SLOT_NAME:
            self.c_slot_type.addItem(slot)
        # Avoid erase all the parameters when navigating though the slots
        self.previous_slot = dict()
        for slot_type in INIT_INDEX:
            self.previous_slot[slot_type] = None

        if self.is_stator:
            self.obj = machine.stator
        else:
            self.obj = machine.rotor

        # If the Slot is not set, initialize it with a 1_0
        if self.obj.slot is None or type(self.obj.slot) is Slot:
            self.obj.slot = SlotW10()
            self.obj.slot._set_None()

        if self.obj.slot.Zs is None:
            self.si_Zs.clear()
        else:
            self.si_Zs.setValue(self.obj.slot.Zs)

        self.set_slot_pitch(self.obj.slot.Zs)

        # Set the correct index for the type checkbox and display the object
        index = INIT_INDEX.index(type(self.obj.slot))
        self.c_slot_type.setCurrentIndex(index)

        # Update the slot widget
        self.s_update_slot()

        # Connect the slot
        self.c_slot_type.currentIndexChanged.connect(self.s_change_slot)
        self.si_Zs.editingFinished.connect(self.set_Zs)
        self.b_plot.clicked.connect(self.s_plot)

    def emit_save(self):
        """Send a saveNeeded signal to the DMachineSetup"""
        self.saveNeeded.emit()

    def set_slot_type(self, index):
        """Initialize self.obj with the slot corresponding to index

        Parameters
        ----------
        self : SWSlot
            A SWSlot object
        index : int
            Index of the selected slot type in the list
        """

        # Save the slot
        self.previous_slot[type(self.obj.slot)] = self.obj.slot

        # Call the corresponding constructor
        Zs = self.obj.slot.Zs
        if self.previous_slot[INIT_INDEX[index]] is None:
            # No previous slot of this type
            self.obj.slot = INIT_INDEX[index]()
            self.obj.slot._set_None()  # No default value
            self.obj.slot.Zs = Zs
        else:  # Load the previous slot of this type
            self.obj.slot = self.previous_slot[INIT_INDEX[index]]
            if self.obj.slot.Zs is not None:
                # Update Zs without trying to compute output
                self.si_Zs.blockSignals(True)
                self.si_Zs.setValue(self.obj.slot.Zs)
                self.si_Zs.blockSignals(False)

                self.set_slot_pitch(self.obj.slot.Zs)

        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_Zs(self):
        """Signal to update the value of Zs according to the spinbox

        Parameters
        ----------
        self : SWSlot
            A SWSlot object
        """
        value = self.si_Zs.value()
        self.obj.slot.Zs = value
        self.set_slot_pitch(value)
        self.w_slot.w_out.comp_output()

        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_slot_pitch(self, Zs):
        """Update out_slot_pitch with the correct value

        Parameters
        ----------
        self : SWSlot
            A SWSlot object
        Zs : int
            The current value of Zs
        """
        sp_txt = self.tr("Slot pitch = 360 / Zs = ")

        if Zs in [None, 0]:
            self.out_Slot_pitch.setText(sp_txt + "?")
        else:
            Slot_pitch = 360.0 / Zs
            Slot_pitch_rad = Slot_pitch * pi / 180

            self.out_Slot_pitch.setText(
                sp_txt
                + "%.4g" % (Slot_pitch)
                + u" [°] ("
                + "%.4g" % (Slot_pitch_rad)
                + " [rad])"
            )

    def s_update_slot(self):
        """Update the slot widget

        Parameters
        ----------
        self : SWSlot
            A SWSlot object
        """

        # Regenerate the pages with the new values
        self.w_slot.setParent(None)
        self.w_slot = WIDGET_LIST[self.c_slot_type.currentIndex()](self.obj)
        self.w_slot.saveNeeded.connect(self.emit_save)
        # Refresh the GUI
        self.main_layout.removeWidget(self.w_slot)
        self.main_layout.insertWidget(1, self.w_slot)

    def s_change_slot(self, index):
        """Signal to update the slot object and widget

        Parameters
        ----------
        self : SWSlot
            A SWSlot object
        index : int
            Current index of the combobox
        """
        # Current slot is removed and replaced by the new one
        self.set_slot_type(index)
        self.s_update_slot()

    def s_plot(self):
        """Try to plot the lamination

        Parameters
        ----------
        self : SWSlot
            A SWSlot object
        """
        # We have to make sure the slot is right before trying to plot it
        error = self.check(self.obj)

        if error:  # Error => Display it
            QMessageBox().critical(self, self.tr("Error"), error)
        else:  # No error => Plot the slot (No winding for LamSquirrelCage)
            if self.machine.type_machine == 10:
                # For SRM, this is the last step => Plot the complete machine
                self.machine.plot()
            else:
                self.obj.plot(is_lam_only=not (type(self.obj) is LamSlotWind))

    @staticmethod
    def check(lam):
        """Check that the current lamination have all the needed field set

        Parameters
        ----------
        lam: LamSlotWind
            Lamination to check

        Returns
        -------
        error: str
            Error message (return None if no error)
        """
        try:
            # Check that everything is set
            if lam.slot.Zs is None:
                return "You must set Zs !"

            # Call the check method of the slot (every slot type have a
            # different check method)
            index = INIT_INDEX.index(type(lam.slot))
            return WIDGET_LIST[index].check(lam)
        except Exception as e:
            return str(e)



# -*- coding: utf-8 -*-

from numpy import pi, sin


def comp_resistance_wind(self, T=20, qs=3):
    """Computation of the equivalent rotor resistance of a cage winding with 'qs' number of phases

    Parameters
    ----------
    self : LamSquirrelCage
        A LamSquirrelCage object
    qs : int
        number of the equivalent rotor winding phases, default value: 3 phases

    Returns
    -------
    Rrot: float
         resistance of the rotor[Ohm]

    """
    # calculate resistance ring
    alpha = self.ring_mat.elec.alpha
    rho20 = self.ring_mat.elec.rho
    rho = rho20 * (1 + alpha * (T - 20))

    Sring = self.comp_surface_ring()
    lring = self.comp_length_ring()
    Zs = self.get_Zs()
    P = self.get_pole_pair_number()

    Rring = rho * lring / Sring

    # calculate resistance rod
    alpha = self.winding.conductor.cond_mat.elec.alpha
    rho20 = self.winding.conductor.cond_mat.elec.rho
    rho = rho20 * (1 + alpha * (T - 20))

    Srod = self.winding.conductor.comp_surface_active()
    lrod = self.comp_length() + 2 * self.winding.Lewout
    Rrod = rho * lrod / Srod

    # calculate total resistance
    Rtot = Zs / qs * (Rrod + Rring / Zs / (2 * sin(pi * P / Zs) ** 2))

    return Rtot



# -*- coding: utf-8 -*-

from numpy import arcsin, exp, angle

from ....Classes.Segment import Segment

from ....Classes.Arc1 import Arc1


def build_geometry(self):
    """Compute the curve (Line) needed to plot the object.
    The ending point of a curve is the starting point of the next curve in
    the list

    Parameters
    ----------
    self : Slot19
        A Slot19 object

    Returns
    -------
    curve_list: list
        A list of 2 Segment and 1 Arc

    """

    [Z1, Z2, Z3, Z4] = self._comp_point_coordinate()

    # Creation of curve
    curve_list = list()
    curve_list.append(Segment(Z1, Z2))
    if self.W1 > 0:
        curve_list.append(Arc1(Z2, Z3, abs(Z3)))
    curve_list.append(Segment(Z3, Z4))

    return curve_list



# -*- coding: utf-8 -*-

# File generated according to PWSlot60.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from ......GUI.Tools.FloatEdit import FloatEdit

from pyleecan.GUI.Resources import pyleecan_rc


class Ui_PWSlot60(object):
    def setupUi(self, PWSlot60):
        if not PWSlot60.objectName():
            PWSlot60.setObjectName(u"PWSlot60")
        PWSlot60.resize(630, 470)
        PWSlot60.setMinimumSize(QSize(630, 470))
        PWSlot60.setMaximumSize(QSize(16777215, 16777215))
        self.horizontalLayout = QHBoxLayout(PWSlot60)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.img_slot = QLabel(PWSlot60)
        self.img_slot.setObjectName(u"img_slot")
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.img_slot.sizePolicy().hasHeightForWidth())
        self.img_slot.setSizePolicy(sizePolicy)
        self.img_slot.setMaximumSize(QSize(16777215, 16777215))
        self.img_slot.setPixmap(
            QPixmap(u":/images/images/MachineSetup/WSlot/Slot_60.PNG")
        )
        self.img_slot.setScaledContents(True)

        self.verticalLayout_2.addWidget(self.img_slot)

        self.txt_constraint = QTextEdit(PWSlot60)
        self.txt_constraint.setObjectName(u"txt_constraint")
        sizePolicy1 = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(
            self.txt_constraint.sizePolicy().hasHeightForWidth()
        )
        self.txt_constraint.setSizePolicy(sizePolicy1)
        self.txt_constraint.setMaximumSize(QSize(16777215, 70))
        self.txt_constraint.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.txt_constraint.setTextInteractionFlags(
            Qt.TextSelectableByKeyboard | Qt.TextSelectableByMouse
        )

        self.verticalLayout_2.addWidget(self.txt_constraint)

        self.horizontalLayout.addLayout(self.verticalLayout_2)

        self.verticalLayout = QVBoxLayout()
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.gridLayout = QGridLayout()
        self.gridLayout.setObjectName(u"gridLayout")
        self.in_R1 = QLabel(PWSlot60)
        self.in_R1.setObjectName(u"in_R1")

        self.gridLayout.addWidget(self.in_R1, 0, 0, 1, 1)

        self.lf_R1 = FloatEdit(PWSlot60)
        self.lf_R1.setObjectName(u"lf_R1")

        self.gridLayout.addWidget(self.lf_R1, 0, 1, 1, 1)

        self.unit_R1 = QLabel(PWSlot60)
        self.unit_R1.setObjectName(u"unit_R1")

        self.gridLayout.addWidget(self.unit_R1, 0, 2, 1, 1)

        self.in_W1 = QLabel(PWSlot60)
        self.in_W1.setObjectName(u"in_W1")

        self.gridLayout.addWidget(self.in_W1, 1, 0, 1, 1)

        self.lf_W1 = FloatEdit(PWSlot60)
        self.lf_W1.setObjectName(u"lf_W1")

        self.gridLayout.addWidget(self.lf_W1, 1, 1, 1, 1)

        self.unit_W1 = QLabel(PWSlot60)
        self.unit_W1.setObjectName(u"unit_W1")

        self.gridLayout.addWidget(self.unit_W1, 1, 2, 1, 1)

        self.in_W2 = QLabel(PWSlot60)
        self.in_W2.setObjectName(u"in_W2")

        self.gridLayout.addWidget(self.in_W2, 2, 0, 1, 1)

        self.lf_W2 = FloatEdit(PWSlot60)
        self.lf_W2.setObjectName(u"lf_W2")

        self.gridLayout.addWidget(self.lf_W2, 2, 1, 1, 1)

        self.unit_W2 = QLabel(PWSlot60)
        self.unit_W2.setObjectName(u"unit_W2")

        self.gridLayout.addWidget(self.unit_W2, 2, 2, 1, 1)

        self.in_H1 = QLabel(PWSlot60)
        self.in_H1.setObjectName(u"in_H1")

        self.gridLayout.addWidget(self.in_H1, 3, 0, 1, 1)

        self.lf_H1 = FloatEdit(PWSlot60)
        self.lf_H1.setObjectName(u"lf_H1")

        self.gridLayout.addWidget(self.lf_H1, 3, 1, 1, 1)

        self.unit_H1 = QLabel(PWSlot60)
        self.unit_H1.setObjectName(u"unit_H1")

        self.gridLayout.addWidget(self.unit_H1, 3, 2, 1, 1)

        self.in_H2 = QLabel(PWSlot60)
        self.in_H2.setObjectName(u"in_H2")

        self.gridLayout.addWidget(self.in_H2, 4, 0, 1, 1)

        self.lf_H2 = FloatEdit(PWSlot60)
        self.lf_H2.setObjectName(u"lf_H2")

        self.gridLayout.addWidget(self.lf_H2, 4, 1, 1, 1)

        self.unit_H2 = QLabel(PWSlot60)
        self.unit_H2.setObjectName(u"unit_H2")

        self.gridLayout.addWidget(self.unit_H2, 4, 2, 1, 1)

        self.in_H3 = QLabel(PWSlot60)
        self.in_H3.setObjectName(u"in_H3")

        self.gridLayout.addWidget(self.in_H3, 5, 0, 1, 1)

        self.lf_H3 = FloatEdit(PWSlot60)
        self.lf_H3.setObjectName(u"lf_H3")

        self.gridLayout.addWidget(self.lf_H3, 5, 1, 1, 1)

        self.unit_H3 = QLabel(PWSlot60)
        self.unit_H3.setObjectName(u"unit_H3")

        self.gridLayout.addWidget(self.unit_H3, 5, 2, 1, 1)

        self.in_H4 = QLabel(PWSlot60)
        self.in_H4.setObjectName(u"in_H4")

        self.gridLayout.addWidget(self.in_H4, 6, 0, 1, 1)

        self.lf_H4 = FloatEdit(PWSlot60)
        self.lf_H4.setObjectName(u"lf_H4")

        self.gridLayout.addWidget(self.lf_H4, 6, 1, 1, 1)

        self.unit_H4 = QLabel(PWSlot60)
        self.unit_H4.setObjectName(u"unit_H4")

        self.gridLayout.addWidget(self.unit_H4, 6, 2, 1, 1)

        self.in_W3 = QLabel(PWSlot60)
        self.in_W3.setObjectName(u"in_W3")

        self.gridLayout.addWidget(self.in_W3, 7, 0, 1, 1)

        self.lf_W3 = FloatEdit(PWSlot60)
        self.lf_W3.setObjectName(u"lf_W3")

        self.gridLayout.addWidget(self.lf_W3, 7, 1, 1, 1)

        self.unit_W3 = QLabel(PWSlot60)
        self.unit_W3.setObjectName(u"unit_W3")

        self.gridLayout.addWidget(self.unit_W3, 7, 2, 1, 1)

        self.verticalLayout.addLayout(self.gridLayout)

        self.g_output = QGroupBox(PWSlot60)
        self.g_output.setObjectName(u"g_output")
        self.g_output.setMinimumSize(QSize(200, 0))
        self.verticalLayout_3 = QVBoxLayout(self.g_output)
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.out_Wlam = QLabel(self.g_output)
        self.out_Wlam.setObjectName(u"out_Wlam")

        self.verticalLayout_3.addWidget(self.out_Wlam)

        self.out_slot_height = QLabel(self.g_output)
        self.out_slot_height.setObjectName(u"out_slot_height")

        self.verticalLayout_3.addWidget(self.out_slot_height)

        self.out_yoke_height = QLabel(self.g_output)
        self.out_yoke_height.setObjectName(u"out_yoke_height")

        self.verticalLayout_3.addWidget(self.out_yoke_height)

        self.out_wind_surface = QLabel(self.g_output)
        self.out_wind_surface.setObjectName(u"out_wind_surface")

        self.verticalLayout_3.addWidget(self.out_wind_surface)

        self.out_tot_surface = QLabel(self.g_output)
        self.out_tot_surface.setObjectName(u"out_tot_surface")

        self.verticalLayout_3.addWidget(self.out_tot_surface)

        self.out_op_angle = QLabel(self.g_output)
        self.out_op_angle.setObjectName(u"out_op_angle")

        self.verticalLayout_3.addWidget(self.out_op_angle)

        self.out_tooth_width = QLabel(self.g_output)
        self.out_tooth_width.setObjectName(u"out_tooth_width")

        self.verticalLayout_3.addWidget(self.out_tooth_width)

        self.verticalLayout.addWidget(self.g_output)

        self.horizontalLayout.addLayout(self.verticalLayout)

        QWidget.setTabOrder(self.lf_R1, self.lf_W1)
        QWidget.setTabOrder(self.lf_W1, self.lf_W2)
        QWidget.setTabOrder(self.lf_W2, self.lf_H1)
        QWidget.setTabOrder(self.lf_H1, self.lf_H2)
        QWidget.setTabOrder(self.lf_H2, self.txt_constraint)

        self.retranslateUi(PWSlot60)

        QMetaObject.connectSlotsByName(PWSlot60)

    # setupUi

    def retranslateUi(self, PWSlot60):
        PWSlot60.setWindowTitle(QCoreApplication.translate("PWSlot60", u"Form", None))
        self.img_slot.setText("")
        self.txt_constraint.setHtml(
            QCoreApplication.translate(
                "PWSlot60",
                u'<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0//EN" "http://www.w3.org/TR/REC-html40/strict.dtd">\n'
                '<html><head><meta name="qrichtext" content="1" /><style type="text/css">\n'
                "p, li { white-space: pre-wrap; }\n"
                "</style></head><body style=\" font-family:'MS Shell Dlg 2'; font-size:7.8pt; font-weight:400; font-style:normal;\">\n"
                '<p align="center" style=" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;"><span style=" font-size:12pt; font-weight:600; text-decoration: underline;">Constraints :</span></p>\n'
                '<p align="center" style=" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;"><span style=" font-size:14pt;">R1 &lt; Rbo</span></p>\n'
                '<p align="center" style=" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;"><span style=" font-size:14pt;">W2&lt; W1</span></p></body></html>',
                None,
            )
        )
        self.in_R1.setText(QCoreApplication.translate("PWSlot60", u"R1 :", None))
        self.unit_R1.setText(QCoreApplication.translate("PWSlot60", u"m", None))
        self.in_W1.setText(QCoreApplication.translate("PWSlot60", u"W1 :", None))
        self.unit_W1.setText(QCoreApplication.translate("PWSlot60", u"m", None))
        self.in_W2.setText(QCoreApplication.translate("PWSlot60", u"W2 :", None))
        self.unit_W2.setText(QCoreApplication.translate("PWSlot60", u"m", None))
        self.in_H1.setText(QCoreApplication.translate("PWSlot60", u"H1 :", None))
        self.unit_H1.setText(QCoreApplication.translate("PWSlot60", u"m", None))
        self.in_H2.setText(QCoreApplication.translate("PWSlot60", u"H2 :", None))
        self.unit_H2.setText(QCoreApplication.translate("PWSlot60", u"m", None))
        self.in_H3.setText(
            QCoreApplication.translate("PWSlot60", u"d_coil_pole_top :", None)
        )
        self.unit_H3.setText(QCoreApplication.translate("PWSlot60", u"m", None))
        self.in_H4.setText(
            QCoreApplication.translate("PWSlot60", u"d_coil_pole_bot :", None)
        )
        self.unit_H4.setText(QCoreApplication.translate("PWSlot60", u"m", None))
        self.in_W3.setText(
            QCoreApplication.translate("PWSlot60", u"d_coil_pole_edg :", None)
        )
        self.unit_W3.setText(QCoreApplication.translate("PWSlot60", u"m", None))
        self.g_output.setTitle(QCoreApplication.translate("PWSlot60", u"Output", None))
        self.out_Wlam.setText(
            QCoreApplication.translate("PWSlot60", u"Lamination width : ?", None)
        )
        self.out_slot_height.setText(
            QCoreApplication.translate("PWSlot60", u"Slot height : ?", None)
        )
        self.out_yoke_height.setText(
            QCoreApplication.translate("PWSlot60", u"Yoke height : ?", None)
        )
        self.out_wind_surface.setText(
            QCoreApplication.translate("PWSlot60", u"Winding surface : ?", None)
        )
        self.out_tot_surface.setText(
            QCoreApplication.translate("PWSlot60", u"Total surface : ?", None)
        )
        self.out_op_angle.setText(
            QCoreApplication.translate("PWSlot60", u"Opening angle : ?", None)
        )
        self.out_tooth_width.setText(
            QCoreApplication.translate("PWSlot60", u"Tooth average width : ?", None)
        )

    # retranslateUi



# -*- coding: utf-8 -*-

import numpy as np


def interpolation(self, point, vertice, field):
    """Return interpolated value of the field in a cell

    Parameters
     ----------
    self : Interpolation
         an Interpolation object
    points : ndarray
        evaluation points
    nb_pt : int
        nb of evaluation points
    vertice : ndarray
        vertices of the cell
    field : ndarray
        field to interpolate

     Returns
     -------
     value: array
         interpolated field

    """

    point_ref = self.get_ref_point(vertice, point)  # TODO: input only single point
    [values_ref, size] = self.shape_function(
        point_ref, 1
    )  # TODO: input only multipel points

    interp_func = np.tensordot(values_ref, field, axes=([2], [0]))

    return interp_func



# -*- coding: utf-8 -*-

from matplotlib.pyplot import axis

from ....Functions.init_fig import init_fig
from ....definitions import config_dict

ROTOR_COLOR = config_dict["PLOT"]["COLOR_DICT"]["ROTOR_COLOR"]
STATOR_COLOR = config_dict["PLOT"]["COLOR_DICT"]["STATOR_COLOR"]


def plot(self, fig=None, ax=None):
    """Plot the Slot in a matplotlib fig

    Parameters
    ----------
    self : Slot
        A Slot object
    fig :
        if None, open a new fig and plot, else add to the current
        one (Default value = None)

    Returns
    -------
    None
    """
    surf = self.get_surface()

    # Display the result
    (fig, axes, patch_leg, label_leg) = init_fig(fig, ax)
    axes.set_xlabel("(m)")
    axes.set_ylabel("(m)")
    axes.set_title("Slot")

    # Add the slot to the fig
    if self.get_is_stator:
        patches = surf.get_patches(color=STATOR_COLOR)
    else:
        patches = surf.get_patch(color=ROTOR_COLOR)
    for patch in patches:
        axes.add_patch(patch)

    # Axis Setup
    axes.axis("equal")
    fig.show()



# -*- coding: utf-8 -*-
from ....Classes.Circle import Circle
from ....Classes.Arc1 import Arc1
from ....Classes.Segment import Segment
from ....Classes.SurfLine import SurfLine
from numpy import exp, pi


def build_geometry(self, sym=1, alpha=0, delta=0):
    """Build the geometry of the shaft

    Parameters
    ----------
    self : Shaft
        Shaft Object
    sym : int
        Symmetry factor (1= full machine, 2= half of the machine...)
    alpha : float
        Angle for rotation [rad]
    delta : complex
        Complex value for translation

    Returns
    -------
    surf_list : list
        list of surfaces needed to draw the lamination

    """
    surf_list = list()

    if sym == 1:
        surf_list.append(
            Circle(radius=self.Drsh / 2, label="Shaft", center=0, point_ref=0)
        )
    else:
        begin = self.Drsh / 2
        end = begin * exp(1j * 2 * pi / sym)
        surface = SurfLine(
            line_list=[
                Segment(0, begin, label="Shaft_Side_Right"),
                Arc1(begin, end, self.Drsh / 2, label="Shaft_Side_Arc"),
                Segment(end, 0, label="Shaft_Side_Left"),
            ],
            label="Shaft",
            point_ref=0,
        )
        surf_list.append(surface)
    for surf in surf_list:
        surf.rotate(alpha)
        surf.translate(delta)
    return surf_list



# -*- coding: utf-8 -*-

# File generated according to PHoleMUD.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from ......GUI.Tools.WPathSelector.WPathSelectorV import WPathSelectorV
from ......GUI.Tools.MPLCanvas import MPLCanvas2
from ......GUI.Dialog.DMatLib.WMatSelect.WMatSelect import WMatSelect

from pyleecan.GUI.Resources import pyleecan_rc


class Ui_PHoleMUD(object):
    def setupUi(self, PHoleMUD):
        if not PHoleMUD.objectName():
            PHoleMUD.setObjectName(u"PHoleMUD")
        PHoleMUD.resize(813, 467)
        PHoleMUD.setMinimumSize(QSize(740, 440))
        PHoleMUD.setMaximumSize(QSize(16777215, 16777215))
        self.horizontalLayout = QHBoxLayout(PHoleMUD)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.w_viewer = MPLCanvas2(PHoleMUD)
        self.w_viewer.setObjectName(u"w_viewer")

        self.horizontalLayout.addWidget(self.w_viewer)

        self.widget = QWidget(PHoleMUD)
        self.widget.setObjectName(u"widget")
        self.widget.setMinimumSize(QSize(250, 0))
        self.widget.setMaximumSize(QSize(250, 16777215))
        self.verticalLayout_2 = QVBoxLayout(self.widget)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.b_dxf = QPushButton(self.widget)
        self.b_dxf.setObjectName(u"b_dxf")

        self.verticalLayout_2.addWidget(self.b_dxf)

        self.w_path_json = WPathSelectorV(self.widget)
        self.w_path_json.setObjectName(u"w_path_json")
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.w_path_json.sizePolicy().hasHeightForWidth())
        self.w_path_json.setSizePolicy(sizePolicy)

        self.verticalLayout_2.addWidget(self.w_path_json)

        self.g_mat = QGroupBox(self.widget)
        self.g_mat.setObjectName(u"g_mat")
        self.g_mat_layout = QVBoxLayout(self.g_mat)
        self.g_mat_layout.setObjectName(u"g_mat_layout")
        self.w_mat_0 = WMatSelect(self.g_mat)
        self.w_mat_0.setObjectName(u"w_mat_0")
        self.w_mat_0.setMinimumSize(QSize(100, 0))

        self.g_mat_layout.addWidget(self.w_mat_0)

        self.verticalLayout_2.addWidget(self.g_mat)

        self.verticalSpacer = QSpacerItem(
            20, 179, QSizePolicy.Minimum, QSizePolicy.Expanding
        )

        self.verticalLayout_2.addItem(self.verticalSpacer)

        self.g_output = QGroupBox(self.widget)
        self.g_output.setObjectName(u"g_output")
        self.g_output.setMinimumSize(QSize(200, 0))
        self.verticalLayout = QVBoxLayout(self.g_output)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.out_slot_surface = QLabel(self.g_output)
        self.out_slot_surface.setObjectName(u"out_slot_surface")

        self.verticalLayout.addWidget(self.out_slot_surface)

        self.out_magnet_surface = QLabel(self.g_output)
        self.out_magnet_surface.setObjectName(u"out_magnet_surface")

        self.verticalLayout.addWidget(self.out_magnet_surface)

        self.out_Rmin = QLabel(self.g_output)
        self.out_Rmin.setObjectName(u"out_Rmin")

        self.verticalLayout.addWidget(self.out_Rmin)

        self.out_Rmax = QLabel(self.g_output)
        self.out_Rmax.setObjectName(u"out_Rmax")

        self.verticalLayout.addWidget(self.out_Rmax)

        self.verticalLayout_2.addWidget(self.g_output)

        self.horizontalLayout.addWidget(self.widget)

        self.retranslateUi(PHoleMUD)

        QMetaObject.connectSlotsByName(PHoleMUD)

    # setupUi

    def retranslateUi(self, PHoleMUD):
        PHoleMUD.setWindowTitle(QCoreApplication.translate("PHoleMUD", u"Form", None))
        self.b_dxf.setText(
            QCoreApplication.translate("PHoleMUD", u"Define Slot from DXF", None)
        )
        self.g_mat.setTitle(QCoreApplication.translate("PHoleMUD", u"Materials", None))
        self.g_output.setTitle(QCoreApplication.translate("PHoleMUD", u"Output", None))
        self.out_slot_surface.setText(
            QCoreApplication.translate("PHoleMUD", u"Hole full surface : ?", None)
        )
        self.out_magnet_surface.setText(
            QCoreApplication.translate("PHoleMUD", u"Hole magnet surface : ?", None)
        )
        self.out_Rmin.setText(QCoreApplication.translate("PHoleMUD", u"Rmin : ?", None))
        self.out_Rmax.setText(QCoreApplication.translate("PHoleMUD", u"Rmax : ?", None))

    # retranslateUi



# -*- coding: utf-8 -*-

from PySide2.QtCore import Qt, Signal
from PySide2.QtGui import QPixmap
from PySide2.QtWidgets import QMessageBox, QWidget

from .....GUI.Dialog.DMachineSetup.SMachineType.Gen_SMachineType import Gen_SMachineType
from .....Classes.Winding import Winding
from .....Classes.MachineIPMSM import MachineIPMSM
from .....Classes.MachineSIPMSM import MachineSIPMSM
from .....Classes.MachineSyRM import MachineSyRM
from .....definitions import PACKAGE_NAME


class SMachineType(Gen_SMachineType, QWidget):
    """First Step to setup the Machine Type"""

    # Signal to DMachineSetup to know that the save popup is needed
    saveNeeded = Signal()
    # Information for the DMachineSetup nav
    step_name = "Machine Type"

    def __init__(self, machine, matlib, is_stator=False):
        """Initialize the widget according to machine

        Parameters
        ----------
        self : SMachineType
            A SMachineType widget
        machine : Machine
            current machine to edit
        matlib : MatLib
            Material Library
        is_stator : bool
            To adapt the GUI to set either the stator or the rotor
        """
        # Build the interface according to the .ui file
        QWidget.__init__(self)
        self.setupUi(self)

        # Saving arguments
        self.machine = machine
        self.matlib = matlib
        self.is_stator = is_stator

        # Dynamic import to avoid import loop
        module = __import__(
            PACKAGE_NAME + ".GUI.Dialog.DMachineSetup", fromlist=["DMachineSetup"]
        )
        self.mach_list = getattr(module, "mach_list")
        self.mach_index = getattr(module, "mach_index")

        # Fill the combobox
        self.c_type.clear()
        self.c_type.addItems(
            [self.mach_dict["name"] for self.mach_dict in self.mach_list]
        )
        # Update the GUI to the current machine type
        index = self.mach_index.index(type(self.machine))
        self.mach_dict = self.mach_list[index]
        self.txt_type_machine.setText(self.mach_dict["txt"])
        self.img_type_machine.setPixmap(QPixmap(self.mach_dict["img"]))
        self.c_type.setCurrentIndex(index)
        if machine.stator.get_pole_pair_number() is not None:
            self.si_p.setValue(machine.stator.winding.p)
        else:
            self.si_p.clear()  # Empty spinbox

        # Set default values
        self.machine.stator.is_stator = True
        self.machine.rotor.is_stator = False

        if machine.rotor.is_internal is None:
            self.machine.rotor.is_internal = True
            self.machine.stator.is_internal = False
            self.is_inner_rotor.setCheckState(Qt.Checked)
        elif machine.rotor.is_internal:
            self.is_inner_rotor.setCheckState(Qt.Checked)
        else:
            self.is_inner_rotor.setCheckState(Qt.Unchecked)

        # WRSM can only have inner rotor
        if self.machine.type_machine == 9:
            self.is_inner_rotor.setEnabled(False)
        else:
            self.is_inner_rotor.setEnabled(True)

        if machine.name not in [None, ""]:
            self.le_name.setText(machine.name)

        # Connect the slot/signal
        self.si_p.editingFinished.connect(self.set_p)
        self.is_inner_rotor.toggled.connect(self.set_inner_rotor)
        self.le_name.editingFinished.connect(self.s_set_name)
        self.c_type.currentIndexChanged.connect(self.set_machine_type)

    def s_set_name(self):
        """Set the name of the machine

        Parameters
        ----------
        self : SMachineType
            A SMachineType object
        """
        self.machine.name = str(self.le_name.text())
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_p(self):
        """Signal to update the value of p according to the spinbox

        Parameters
        ----------
        self : SMachineType
            A SMachineType object
        """
        value = self.si_p.value()
        if self.machine.stator.winding is None:
            self.machine.stator.winding = Winding()
            self.machine.stator.winding._set_None()
        self.machine.stator.winding.p = value
        # Set machine rotor p according to machine type
        if type(self.machine) is MachineSIPMSM:
            self.machine.rotor.slot.Zs = 2 * value
        elif type(self.machine) in [MachineIPMSM, MachineSyRM]:
            if self.machine.rotor.hole is None:
                self.machine.rotor.hole = list()
            else:
                for hole in self.machine.rotor.hole:
                    hole.Zh = 2 * value
        elif self.machine.type_machine == 10:
            pass
        else:
            self.machine.rotor.winding.p = value

        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_inner_rotor(self, is_checked):
        """Signal to update the value of is_internal according to the widget

        Parameters
        ----------
        self : SMachineType
            A SMachineType object
        is_checked : bool
            State of is_internal
        """

        self.machine.stator.is_internal = not is_checked
        self.machine.rotor.is_internal = is_checked
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_machine_type(self, index):
        """Change the machine type according to the combobox

        Parameters
        ----------
        self : SMachineType
            A SMachineType object
        index : int
            Selected machine type index
        """
        p = self.machine.stator.get_pole_pair_number()
        # Get the correct machine class
        mach = self.mach_list[index]["init_machine"]
        self.machine = type(mach)(init_dict=mach.as_dict())
        if p is not None:
            self.si_p.setValue(p)
            self.set_p()
        # Update the GUI with the new machine
        self.parent().machine = self.machine
        self.parent().update_nav(next_step=0)
        if self.parent() is not None:
            self.parent().main_layout.removeWidget(self)

    @staticmethod
    def check(machine):
        """Check that the current machine have all the needed field set

        Parameters
        ----------
        machine : Machine
            Machine to check

        Returns
        -------
        error: str
            Error message (return None if no error)
        """
        try:
            if machine.stator.winding is None:
                return "Missing stator winding"
            if machine.stator.get_pole_pair_number() in [None, 0]:
                return "p must be >0 !"
        except Exception as e:
            return str(e)



import itertools
import numpy as np
from ....Classes.ParamExplorerSet import ParamExplorerSet
from ....Functions.Simulation.VarLoad.setter_simu import setter_simu


def generate_simulation_list(self, ref_simu=None):
    """Generate all the simulation for the multi-simulation

    Parameters
    ----------
    self : VarSimu
        A VarSimu object
    ref_simu : Simulation
        Reference simulation to copy / update

    Returns
    -------
    multisim_dict : dict
        dictionary containing the simulation and paramexplorer list
    """

    # Get InputCurrent list
    list_input = self.get_input_list()

    multisim_dict = {
        "paramexplorer_list": [],  # Setter's values
        "simulation_list": [],
    }

    # Create Simulations 1 per load
    for input_obj in list_input:
        # Generate the simulation
        new_simu = ref_simu.copy(keep_function=True)

        # Edit simulation
        # setter current
        setter_simu(new_simu, input_obj)
        # Add simulation to the list
        multisim_dict["simulation_list"].append(new_simu)

    # Create ParamExplorerSet
    #   This version uses a single ParamExplorerSet to define the simulation
    #   Other parameters can be stored in a dedicated ParamExplorerSet if needed
    multisim_dict["paramexplorer_list"].append(
        ParamExplorerSet(
            name="InputCurrent",
            symbol="",
            unit="",
            setter=setter_simu,
            value=list_input,
        )
    )

    # Generate default datakeeper
    self.gen_datakeeper_list()

    return multisim_dict



# -*- coding: utf-8 -*-

import numpy as np


def find_cell(self, points, nb_pt, normal_t=None):
    """Return the cells containing the target point(s)

    Parameters
    ----------
    self : MeshMat
        an MeshMat object
    points : ndarray
        coordinates of the target point(s)
    nb_pt : int
        number of target points

    Returns
    -------
    cell_list: list
        A list of  of selected cells

    """
    # nmax_search = 3
    cells_list = list()
    cell_prop = list()

    point = self.point
    point_coord = point.coordinate
    nb_tot_pt = point.nb_pt

    for ii in range(nb_pt):
        if nb_pt == 1:
            pt = points
        else:
            pt = points[ii, :]
        for key in self.cell:
            cells = self.cell[key]
            ref_cell = cells.interpolation.ref_cell
            connect = cells.connectivity
            nb_pt_per_cell = cells.nb_pt_per_cell
            # vertice0 = point_coord[connect[0]]
            # dist_ref = np.sqrt(
            #     np.square(vertice0[0, 0] - vertice0[1, 0])
            #     + np.square(vertice0[0, 1] - vertice0[1, 1])
            # )

            # compute the distance of all nodes to the current point 'pt'
            point_rep = np.tile(pt, (nb_tot_pt, 1))

            if self.dimension == 3:
                dist_node = np.reshape(
                    np.sqrt(
                        np.square(point_coord[:, 0] - point_rep[:, 0])
                        + np.square(point_coord[:, 1] - point_rep[:, 1])
                        + np.square(point_coord[:, 2] - point_rep[:, 2])
                    ),
                    (nb_tot_pt, 1),
                )
            else:
                dist_node = np.reshape(
                    np.sqrt(
                        np.square(point_coord[:, 0] - point_rep[:, 0])
                        + np.square(point_coord[:, 1] - point_rep[:, 1])
                    ),
                    (nb_tot_pt, 1),
                )
            # min_dist = np.min(dist_node)
            # min_node = np.where(dist_node == min_dist)[0]
            Imin_node = np.argsort(dist_node, axis=0)
            # Imin_node = Imin_node[0:nmax_search]  #

            # All selected nodes from the closest to the farthest are tested
            inode = 0
            # dontstop = True

            # while inode < nmax_search and inode < nb_tot_pt and dontstop:

            # get cells that contain the closest node and test if point is inside
            closest_cells = np.where(connect == Imin_node[inode])[0]
            nb_closest_elem = len(closest_cells)
            a, b = np.zeros(nb_closest_elem), np.zeros(nb_closest_elem)
            cell_prop = list()
            for ielt in range(nb_closest_elem):
                vert = self.get_vertice(closest_cells[ielt])[key]
                (is_inside, a[ielt], b[ielt]) = ref_cell.is_inside(vert, pt, normal_t)
                if is_inside:
                    # dontstop = False
                    cell_prop.append(key)
                    cell_prop.append(closest_cells[ielt])

                # inode = inode + 1
                # break

            # if no cell was found, give it a second try
            # TODO first check if outside mesh
            if len(cell_prop) == 0:
                # test all cells (sorted by center, i.e. mean of vertices)
                vert_cent = np.zeros(pt.shape)
                for icell in range(nb_pt_per_cell):
                    vert_cent = (
                        vert_cent + point_coord[connect[:, icell]] / nb_pt_per_cell
                    )

                if self.dimension == 3:
                    dist_vert_cent = np.reshape(
                        np.sqrt(
                            (vert_cent[:, 0] - pt[0]) ** 2
                            + (vert_cent[:, 1] - pt[1]) ** 2
                            + (vert_cent[:, 2] - pt[2]) ** 2
                        ),
                        (cells.nb_cell, 1),
                    )
                else:
                    dist_vert_cent = np.reshape(
                        np.sqrt(
                            (vert_cent[:, 0] - pt[0]) ** 2
                            + (vert_cent[:, 1] - pt[1]) ** 2
                        ),
                        (cells.nb_cell, 1),
                    )

                Imin_vert_cent = np.argsort(dist_vert_cent, axis=0)[:, 0]
                i = 0
                is_inside = False
                while i < cells.nb_cell and not is_inside:
                    vert = self.get_vertice(Imin_vert_cent[i])[key]
                    (is_inside, a, b) = ref_cell.is_inside(vert, pt, normal_t)
                    if is_inside:
                        # dontstop = False
                        cell_prop = [key, Imin_vert_cent[i]]
                    i += 1

            # No cell contain the point atleast (only possible if point is outside mesh)
            if len(cell_prop) == 0:
                cells_list.append(None)
            # one cell found
            elif len(cell_prop) == 2:
                cells_list.append(cell_prop)
            # more than one cells found
            elif len(cell_prop) > 2:
                ind = np.where(np.min(a) == a)[0][0]
                cells_list.append(cell_prop[2 * ind : 2 * ind + 2])

    return cells_list



# -*- coding: utf-8 -*-


def get_Ryoke(self):
    """Return the yoke lamination radius

    Parameters
    ----------
    self : Lamination
        A Lamination object

    Returns
    -------
    Ryoke: float
        The lamination yoke radius [m]

    """

    if self.is_internal:
        return self.Rint
    else:
        return self.Rext



# -*- coding: utf-8 -*-
from matplotlib.patches import Polygon

from ....definitions import config_dict

PATCH_EDGE = config_dict["PLOT"]["COLOR_DICT"]["PATCH_EDGE"]
PATCH_COLOR = config_dict["PLOT"]["COLOR_DICT"]["PATCH_COLOR"]
PATCH_COLOR_ALPHA = config_dict["PLOT"]["COLOR_DICT"]["PATCH_COLOR_ALPHA"]
PATCH_EDGE_ALPHA = config_dict["PLOT"]["COLOR_DICT"]["PATCH_EDGE_ALPHA"]


def get_patches(
    self, color=PATCH_COLOR, edgecolor=PATCH_EDGE, is_edge_only=False, linestyle=None
):
    """Returns the PolarArc Patch to be display in matplotlib
    Parameters
    ----------
    self : PolarArc
        a PolarArc object
    color :
        The color of the patch (Default value = PATCH_COLOR)
    edgecolor :
        The color of the edgecolor (Default value = PATCH_EDGE)
    is_edge_only: bool
        To set the transparancy of the face color to 0 and 1 for the edge color
    linestyle : str
        Line style of the edge {'-', '--', '-.', ':', '', (offset, on-off-seq), ...}

    Returns
    -------
    patch_list : list of matplotlib.patches.Polygon
        List of patches corresponding to the surface
    """

    # check if the PolarArc is correct
    self.check()

    if is_edge_only:
        color = PATCH_COLOR_ALPHA
        edgecolor = PATCH_EDGE_ALPHA

    line_list = self.get_lines()
    Z_list = list()
    # For each  Line discretize
    for line in line_list:
        Z_list.extend(line.discretize())

    # abscissa coordinate
    Zr_list = list()
    # ordinate coordinate
    Zi_list = list()

    for ii in range(len(Z_list)):
        Zr_list.append(Z_list[ii].real)
        Zi_list.append(Z_list[ii].imag)
    point_list = list(zip(Zr_list, Zi_list))
    return [
        Polygon(point_list, facecolor=color, edgecolor=edgecolor, linestyle=linestyle)
    ]



# -*- coding: utf-8 -*-

from numpy import pi


def comp_surface_active(self):
    """Compute the active surface of the conductor

    Parameters
    ----------
    self : CondType12
        A CondType12 object

    Returns
    -------
    Sact: float
        Surface without insulation [m**2]

    """

    Sact = ((self.Wwire / 2.0) ** 2) * pi * self.Nwppc

    return Sact



# -*- coding: utf-8 -*-
from numpy import pi, exp, linspace

from ....Classes.Arc1 import Arc1
from ....Classes.Arc3 import Arc3
from ....Methods import NotImplementedYetError


def get_bore_line(self, alpha1, alpha2, label="", ignore_notches=False):
    """

    Parameters
    ----------
    self : Lamination
        a Lamination object
    alpha1 : float
        Starting angle [rad]
    alpha2 : float
        Ending angle [rad]
    label : str
        the label of the bore line

    Returns
    -------
    bore_line : list
        list of bore line

    """
    delta = alpha2 - alpha1

    if delta == 0:
        return []

    elif delta > 2 * pi:
        raise NotImplementedYetError(
            "Only angle smaller/equal to 2*pi are implemented."
        )
    else:
        # ignore_notches = 1
        if not self.notch or ignore_notches:
            Rbo = self.get_Rbo()
            Z1 = Rbo * exp(1j * alpha1)
            if delta == 2 * pi:
                Z2 = Rbo * exp(1j * (alpha1 + delta / 2))
                line1 = Arc3(begin=Z1, end=Z2, is_trigo_direction=True, label=label)
                line2 = Arc3(begin=Z2, end=Z1, is_trigo_direction=True, label=label)
                return [line1, line2]
            else:
                Z2 = Rbo * exp(1j * alpha2)
                return [Arc1(begin=Z1, end=Z2, radius=Rbo, label=label)]
        else:
            # === intermediate solution ========================================
            # no check for intersection of different notches
            line_list = self.notch[0].build_geometry(alpha1, alpha2, label=label)
            # === later =======================================================
            """
            bore_line = self.get_bore_line(alpha1, alpha2, ignore_notches=True)
            for notch in notch_shape:
                notch_line = notch.build_geometry()
                bore_line = build_intersect_geometry(bore_line, notch_line,
                                            inwards=True)
            """

            return line_list



from numpy import hstack, vstack, zeros, savetxt
from PySide2.QtWidgets import QDialog, QFileDialog
from pandas import read_csv
from ....GUI.Tools.FloatEdit import FloatEdit
from ....GUI.Tools.WTableData.Ui_DTableData import Ui_DTableData
from os.path import isfile
from PySide2.QtWidgets import QMessageBox


class DTableData(Ui_DTableData, QDialog):
    def __init__(
        self,
        parent=None,
        data=None,
        Vmin=-999999999,
        Vmax=999999999,
        title=None,
        shape_min=(1, 1),
        shape_max=(None, None),
        col_header=None,
    ):
        """Initialization of the widget

        Parameters
        ----------
        data : ndarray
            Data to display/edit
        Vmin : float
            Minimum value of the data
        Vmax : float
            Maximum value of the data
        title : str
            Title for the Dialog window
        shape_min : list
            [Min row number, Min column number], None for "no" constraint
        shape_max : list
            [Max row number, Max column number], None for "no" constraint
        col_header : list
            Provide Header to the columns (None to remove header)
        """
        QDialog.__init__(self, parent)
        self.setupUi(self)
        self.data = data if data is not None else zeros((1, 2))
        self.Vmin = Vmin
        self.Vmax = Vmax
        self.title = title  # To change the dialog title
        self.shape_min = shape_min
        self.shape_max = shape_max
        self.col_header = col_header
        self.N_row_txt = "N_row"
        self.N_col_txt = "N_column"
        # Import/Export Hidden by default, need to be shown in widget definition
        self.b_import.hide()
        self.b_export.hide()

        self.update()
        self.si_row.valueChanged.connect(self.set_Nrow)
        self.si_col.valueChanged.connect(self.set_Ncol)
        self.b_export.clicked.connect(self.export_csv)
        self.b_import.clicked.connect(self.import_csv)

    def update(self):
        """Update the Widget according to the current data"""
        self.si_row.setValue(self.data.shape[0])
        self.si_col.setValue(self.data.shape[1])
        self.in_col.setText(self.N_col_txt)
        self.in_row.setText(self.N_row_txt)

        # Set row shape limit
        Rmin = 1
        Rmax = 999999999
        if self.shape_min[0] is not None:
            Rmin = max(Rmin, self.shape_min[0])
        if self.shape_max[0] is not None and self.shape_max[0] >= Rmin:
            Rmax = min(Rmax, self.shape_max[0])
        self.si_row.setMinimum(Rmin)
        self.si_row.setMaximum(Rmax)

        if Rmin == Rmax:
            self.si_row.setValue(Rmin)
            self.set_Nrow()
            self.si_row.setEnabled(False)
        else:
            self.si_row.setEnabled(True)

        # Set column shape limit
        Cmin = 1
        Cmax = 999999999
        if self.shape_min[1] is not None:
            Cmin = max(Cmin, self.shape_min[1])
        if self.shape_max[1] is not None and self.shape_max[1] >= Cmin:
            Cmax = min(Cmax, self.shape_max[1])
        self.si_col.setMinimum(Cmin)
        self.si_col.setMaximum(Cmax)

        if Cmin == Cmax:
            self.si_col.setValue(Cmin)

            self.si_col.setEnabled(False)
        else:
            self.si_col.setEnabled(True)

        # Set title
        if self.title is not None:
            self.setWindowTitle(self.title)

        # Fill the table
        self.update_tab()

    def update_tab(self):
        """Update the Table to display the current data + setup edit"""
        self.w_tab.clear()
        self.w_tab.setRowCount(self.data.shape[0])
        self.w_tab.setColumnCount(self.data.shape[1])

        # Set column header
        if self.col_header is not None:
            self.w_tab.setHorizontalHeaderLabels(self.col_header)

        for ii in range(self.data.shape[0]):
            for jj in range(self.data.shape[1]):
                self.w_tab.setCellWidget(ii, jj, FloatEdit())
                # Add the min/max to the widget
                self.w_tab.cellWidget(ii, jj).validator().setBottom(self.Vmin)
                self.w_tab.cellWidget(ii, jj).validator().setTop(self.Vmax)
                # Set the values
                self.w_tab.cellWidget(ii, jj).setValue(self.data[ii, jj])
                # Connect the slot
                # self.w_tab.cellWidget(ii, jj).editingFinished.connect(self.set_data)

    def set_Nrow(self):
        """Change the number of row"""
        Nrow_1 = self.data.shape[0]
        Nrow_2 = self.si_row.value()

        if Nrow_1 < Nrow_2:
            # Add some new lines
            data = zeros((Nrow_2, self.data.shape[1]))
            data[:Nrow_1, :] = self.data
            self.data = data
            self.update_tab()
        elif Nrow_1 > Nrow_2:
            # Remove the last lines
            self.data = self.data[:Nrow_2, :]
            self.update_tab()
        # Else do nothing

    def set_Ncol(self):
        """Change the number of column"""
        Ncol_1 = self.data.shape[1]
        Ncol_2 = self.si_col.value()

        if Ncol_1 < Ncol_2:
            # Add some new columns
            data = zeros((self.data.shape[0], Ncol_2))
            data[:, :Ncol_1] = self.data
            self.data = data
            self.update_tab()
        elif Ncol_1 > Ncol_2:
            # Remove the last columns
            self.data = self.data[:, :Ncol_2]
            self.update_tab()
        # Else do nothing

    def get_data(self):
        """Return the current data matrix"""
        data = zeros((self.si_row.value(), self.si_col.value()))
        for ii in range(self.si_row.value()):
            for jj in range(self.si_col.value()):
                data[ii, jj] = self.w_tab.cellWidget(ii, jj).value()
        return data

    def export_csv(self):
        """Write the current table data into a csv file"""
        data = self.get_data()
        save_file_path = QFileDialog.getSaveFileName(
            self, self.tr("Save file"), "", "CSV (*.csv)"
        )[0]
        if save_file_path not in [None, ""]:
            savetxt(save_file_path, data, delimiter=",")

    def import_csv(self):
        """Import the data from a csv file and update the tavle"""
        load_path = str(
            QFileDialog.getOpenFileName(
                self, self.tr("Load CSV file"), "", "CSV (*.csv)"
            )[0]
        )
        if isfile(load_path):
            df = read_csv(
                load_path,
                header=None,
            )
            if type(df.values[0, 0]) is str:
                # Load remove first line
                df = read_csv(
                    load_path,
                )
            val = df.values
            # Check Shape
            error = ""
            shape = val.shape
            if self.shape_min:
                if self.shape_min[0] and shape[0] < self.shape_min[0]:
                    error = "Minimum number of line should be " + str(self.shape_min[0])
                if self.shape_min[1] and shape[1] < self.shape_min[1]:
                    error = "Minimum number of column should be " + str(
                        self.shape_min[1]
                    )
            if self.shape_max:
                if self.shape_max[0] and shape[0] > self.shape_max[0]:
                    error = "Maximum number of line should be " + str(self.shape_max[0])
                if self.shape_max[1] and shape[1] > self.shape_max[1]:
                    error = "Maximum number of column should be " + str(
                        self.shape_max[1]
                    )
            if error:
                QMessageBox().critical(
                    self,
                    self.tr("Error"),
                    self.tr("Error while loading csv file:\n" + error),
                )
                return
            # Try to fill the table
            try:
                old_data = self.data
                self.data = val
                self.update()
            except Exception as e:
                QMessageBox().critical(
                    self,
                    self.tr("Error"),
                    self.tr("Error while loading csv file:\n" + str(e)),
                )

    def accept(self):
        """Update the data before closing"""
        self.data = self.get_data()
        super().accept()



# -*- coding: utf-8 -*-
from numpy import pi, arcsin


def comp_length(self):
    """Compute the length of the arc

    Parameters
    ----------
    self : Arc1
        An Arc1 object

    Returns
    -------
    length: float
        length of the arc
    """

    self.check()

    R = abs(self.radius)

    # alpha is the opening angle
    alpha = abs(self.get_angle())

    # 2*pi*R is the length of the total circle
    # The arc is an alpha / 2pi portion of the cercle
    return float(2 * pi * R * (alpha / (2 * pi)))



from ....Methods.Slot.Slot import SlotCheckError


class S11_W01CheckError(SlotCheckError):
    """ """

    pass


class S11_RWCheckError(SlotCheckError):
    """ """

    pass


class S11_RHCheckError(SlotCheckError):
    """ """

    pass


class S11_H1rCheckError(SlotCheckError):
    """ """

    pass



# -*- coding: utf-8 -*-

from numpy import pi
from ....Classes.LamSlot import LamSlot


def comp_volumes(self):
    """Compute the Lamination volumes (Vlam, Vvent, Vslot, Vwind)

    Parameters
    ----------
    self : LamSlotWind
        A LamSlotWind object

    Returns
    -------
    V_dict: dict
        Lamination volume dictionnary (Vlam, Vvent, Vslot, Vwind) [m**3]

    """

    V_dict = LamSlot.comp_volumes(self)
    Lf = self.comp_length()  # Include radial ventilation ducts
    if self.slot is None:
        V_dict["Vwind"] = 0
    else:
        V_dict["Vwind"] = Lf * self.get_Zs() * self.slot.comp_surface_active()

    return V_dict



from numpy import angle, arcsin, arctan, cos, exp, pi, sin, sqrt

from ....Methods.Slot.SlotW15 import S15InnerError


def _comp_point_coordinate(self):
    """Compute the point coordinates needed to plot the Slot.

    Parameters
    ----------
    self : SlotW15
        A SlotW15 object

    Returns
    -------
    point_dict: dict
        A dict of the slot point coordinates
    """
    Rbo = self.get_Rbo()

    # alpha is the angle to rotate Z0 so ||Z1,Z13|| = W0
    alpha = arcsin(self.W0 / (2 * Rbo))
    hsp = pi / self.Zs  # Half slot pitch

    Z0 = Rbo * exp(1j * 0)
    Z13 = Z0 * exp(1j * alpha)
    if self.is_outwards():
        Z12 = Z0 + self.H0 + 1j * Z13.imag
        Z7 = Z0 + self.H0 + self.H1 + self.H2

        # Zc3, Z8 and (0,0) are align (tangent) => abs(Zc3)=Rbo+H0+H1+H2-R2
        # In tooth ref: Im(Zc3') = -W3/2 - R2
        A = Rbo + self.H0 + self.H1 + self.H2 - self.R2
        B = -self.W3 / 2 - self.R2
        xc2 = B * sin(-hsp) + sqrt(-(B ** 2) + A ** 2) * cos(-hsp)
        yc2 = B * cos(-hsp) - sqrt(-(B ** 2) + A ** 2) * sin(-hsp)
        Zc3 = xc2 + 1j * yc2
        Z9 = (Zc3 * exp(1j * -hsp) + self.R2 * 1j) * exp(1j * hsp)

        # Zc3, Z8 and (0,0) are align, |Zc3, Z8| = R2
        Z8 = (Zc3 * exp(1j * -angle(Zc3)) + self.R2) * exp(1j * angle(Zc3))

        # Real(Zc4) = Rbo+H0+H1
        # In tooth ref: Im(Zc4') = -W3/2 - R1
        xc1 = Rbo + self.H0 + self.H1
        yc1 = (-self.W3 / 2 - self.R1 - xc1 * sin(-hsp)) / cos(-hsp)
        Zc4 = xc1 + 1j * yc1
        Z10 = (Zc4 * exp(1j * -hsp) + self.R1 * 1j) * exp(1j * hsp)

        # Ref center at Zc4, (Z11,Z12) and (Z11,Zc4) are orthogonal
        # Z11 = R1*exp(1i*theta)
        # (R1*cos(theta)-x2)*R1*cos(theta)+(R1*sin(theta)-y2)*R1*sin(theta) = 0
        R1 = self.R1
        y2 = (Z12 - Zc4).imag
        x2 = (Z12 - Zc4).real
        theta = 2 * arctan((y2 - sqrt(-(R1 ** 2) + x2 ** 2 + y2 ** 2)) / (R1 + x2))
        Z11 = R1 * exp(1j * theta) + Zc4
    else:
        raise S15InnerError("Slot Type 15 can't be used on inner lamination")

    point_dict = dict()
    # symetry
    point_dict["Z13"] = Z13
    point_dict["Z12"] = Z12
    point_dict["Z11"] = Z11
    point_dict["Z10"] = Z10
    point_dict["Z9"] = Z9
    point_dict["Z8"] = Z8
    point_dict["Z7"] = Z7
    point_dict["Z6"] = Z8.conjugate()
    point_dict["Z5"] = Z9.conjugate()
    point_dict["Z4"] = Z10.conjugate()
    point_dict["Z3"] = Z11.conjugate()
    point_dict["Z2"] = Z12.conjugate()
    point_dict["Z1"] = Z13.conjugate()
    point_dict["Zc4"] = Zc4
    point_dict["Zc3"] = Zc3
    point_dict["Zc2"] = Zc3.conjugate()
    point_dict["Zc1"] = Zc4.conjugate()
    return point_dict



# -*- coding: utf-8 -*-

from numpy import arcsin, pi

from ....Methods.Slot.Slot import SlotCheckError
from ....Methods.Slot.HoleM58 import *


def check(self):
    """Check that the HoleM58 object is correct

    Parameters
    ----------
    self : HoleM58
        A HoleM58 object

    Returns
    -------
    None

    Raises
    _______
    S58_WCheckError
        You must have W1+W2 <= W0
    """
    # Check that everything is set
    if self.W0 is None:
        raise S58_NoneError("You must set W0 !")
    elif self.W1 is None:
        raise S58_NoneError("You must set W1 !")
    elif self.W2 is None:
        raise S58_NoneError("You must set W2 !")
    elif self.W3 is None:
        raise S58_NoneError("You must set W3 !")
    elif self.R0 is None:
        raise S58_NoneError("You must set R0 !")
    elif self.H0 is None:
        raise S58_NoneError("You must set H0 !")
    elif self.H1 is None:
        raise S58_NoneError("You must set H1 !")
    elif self.H2 is None:
        raise S58_NoneError("You must set H2 !")

    Rext = self.get_Rext()

    if self.W0 < self.W1 + self.W2:
        raise S58_WCheckError("You must have W1+W2 <= W0")



from ....Classes.Arc2 import Arc2
from numpy import pi


def split_half(self, is_begin=True):
    """Cut the line in half (modify the object Arc3 => Arc2)

    Parameters
    ----------
    self : Arc3
        An Arc3 object
    is_begin : bool
        True to keep the part begin=>middle, False for the part middle=>end

    Returns
    -------
    """

    if self.is_trigo_direction:
        sign = 1
    else:
        sign = -1

    if is_begin:
        arc = Arc2(begin=self.begin, center=self.get_center(), angle=sign * pi / 2)
    else:
        arc = Arc2(
            begin=self.get_middle(), center=self.get_center(), angle=sign * pi / 2
        )
    # Change the object type from Arc3 => Arc2
    self.__class__ = Arc2
    self.__dict__.update(arc.__dict__)



from numpy import pi, array, transpose

from SciDataTool import Data1D, DataTime

from ....Functions.Electrical.coordinate_transformation import dq2n
from ....Functions.Winding.gen_phase_list import gen_name


def get_Is(self):
    """Return the stator current DataTime object

    Parameters
    ----------
    self : OutElec
        an OutElec object

    """
    # Calculate stator currents if Is is not in OutElec
    if self.Is is None:
        # Generate current according to Id/Iq
        Isdq = array([self.Id_ref, self.Iq_ref])
        time = self.Time.get_values(is_oneperiod=True)
        qs = self.parent.simu.machine.stator.winding.qs
        felec = self.felec

        # Get rotation direction of the fundamental magnetic field created by the winding
        rot_dir = self.parent.get_rot_dir()

        # Get stator current function of time
        Is = dq2n(Isdq, 2 * pi * felec * time, n=qs, rot_dir=rot_dir, is_n_rms=False)

        Phase = Data1D(
            name="phase",
            unit="",
            values=gen_name(qs),
            is_components=True,
        )
        self.Is = DataTime(
            name="Stator current",
            unit="A",
            symbol="Is",
            axes=[Phase, self.Time.copy()],
            values=transpose(Is),
        )
    return self.Is



from ....Methods.Slot.SlotW16 import S16OutterError


def check(self):
    """Check that the SlotW16 object is correct

    Parameters
    ----------
    self : SlotW16
        A SlotW16 object

    Returns
    -------
    None

    Raises
    -------
    S16OutterError
        Slot 16 must be on internal lamination only
    """
    if self.is_outwards():
        raise S16OutterError("Slot 16 must be on internal lamination only")



# -*- coding: utf-8 -*-
from numpy import take, where, array


def get_solution(self, indice=None):
    """Return a copy of the solution with the option to only include specified indices.

    Parameters
    ----------
    self : SolutionMat
        a SolutionMat object
    indices : list
        list of indices, if list is empty or None all indices are included

    Returns
    -------
    solution: SolutionMat
        solution

    """
    logger = self.get_logger()

    field_sol = self.get_field()
    axis_name, axis_size = self.get_axes_list()
    s_indice = self.indice

    Iindice = axis_name.index("indice")

    # create indices of solution if None
    if s_indice is None:
        s_indice = [i for i in range(field_sol.shape[Iindice])]

    # check input indices
    if not indice:
        indice = s_indice

    if set(indice) - set(s_indice):
        logger.warning(
            "At least one input indice is not part of the solution. "
            + "Respective indice will be skipped."
        )

    # skip indice that are not part of the solution
    new_indice = [ii for ii in indice if ii in s_indice]

    # get array index of new_indice
    array_indice = [where(array(s_indice) == ii)[0][0] for ii in new_indice]

    # setup requested solution
    axis_size[Iindice] = len(new_indice)
    new_field_sol = take(field_sol, array_indice, axis=Iindice)

    solution = type(self)(
        label=self.label,
        type_cell=self.type_cell,
        field=new_field_sol,
        indice=indice,
        axis_name=axis_name,
        axis_size=axis_size,
        dimension=self.dimension,
    )

    return solution



# -*- coding: utf-8 -*-

from itertools import repeat

import matplotlib.pyplot as plt
from numpy import argmin, abs, squeeze, split, ndarray

from ...Functions.init_fig import init_fig
from ...definitions import config_dict

# Import values from config dict
FONT_NAME = config_dict["PLOT"]["FONT_NAME"]
FONT_SIZE_TITLE = config_dict["PLOT"]["FONT_SIZE_TITLE"]
FONT_SIZE_LABEL = config_dict["PLOT"]["FONT_SIZE_LABEL"]
FONT_SIZE_LEGEND = config_dict["PLOT"]["FONT_SIZE_LEGEND"]


def plot_2D(
    Xdatas,
    Ydatas,
    legend_list=[""],
    color_list=[(0, 0, 1, 0.5)],
    linestyle_list=["-"],
    linewidth_list=[2],
    title="",
    xlabel="",
    ylabel="",
    fig=None,
    ax=None,
    is_logscale_x=False,
    is_logscale_y=False,
    is_disp_title=True,
    is_grid=True,
    type_plot="curve",
    fund_harm=None,
    x_min=None,
    x_max=None,
    y_min=None,
    y_max=None,
    xticks=None,
    save_path=None,
    barwidth=100,
    is_show_fig=None,
    win_title=None,
):
    """Plots a 2D graph (curve, bargraph or barchart) comparing fields in Ydatas

    Parameters
    ----------
    Xdatas : ndarray
        array of x-axis values
    Ydatas : list
        list of y-axes values
    legend_list : list
        list of legends
    color_list : list
        list of colors to use for each curve
    linewidth_list : list
        list of line width to use for each curve
    title : str
        title of the graph
    xlabel : str
        label for the x-axis
    ylabel : str
        label for the y-axis
    fig : Matplotlib.figure.Figure
        existing figure to use if None create a new one
    ax : Matplotlib.axes.Axes object
        ax on which to plot the data
    is_logscale_x : bool
        boolean indicating if the x-axis must be set in logarithmic scale
    is_logscale_y : bool
        boolean indicating if the y-axis must be set in logarithmic scale
    is_disp_title : bool
        boolean indicating if the title must be displayed
    is_grid : bool
        boolean indicating if the grid must be displayed
    type_plot : str
        type of 2D graph : "curve", "bargraph", "barchart" or "quiver"
    fund_harm : float
        frequency/order/wavenumber of the fundamental harmonic that must be displayed in red in the fft
    x_min : float
        minimum value for the x-axis
    x_max : float
        maximum value for the x-axis
    y_min : float
        minimum value for the y-axis
    y_max : float
        maximum value for the y-axis
    xticks : list
        list of ticks to use for the x-axis
    save_path : str
        full path including folder, name and extension of the file to save if save_path is not None
    barwidth : float
        barwidth scaling factor, only if type_plot = "bargraph"
    is_show_fig : bool
        True to show figure after plot
    win_title : str
        Title of the plot window
    """

    # Set is_show_fig if is None
    if is_show_fig is None:
        is_show_fig = True if fig is None else False

    # Set figure if needed
    if fig is None and ax is None:
        (fig, ax, _, _) = init_fig(fig=None, shape="rectangle")

    # Number of curves on a axe
    ndatas = len(Ydatas)

    # Retrocompatibility
    if isinstance(Xdatas, ndarray):
        Xdatas = [Xdatas]

    if len(Xdatas) == 1:
        i_Xdatas = [0 for i in range(ndatas)]
    else:
        i_Xdatas = range(ndatas)

    # Expend default argument
    if 1 == len(color_list) < ndatas:
        # Set the same color for all curves
        color_list = list(repeat(color_list[0], ndatas))
    if 1 == len(linewidth_list) < ndatas:
        # Set the same color for all curves
        linewidth_list = list(repeat(linewidth_list[0], ndatas))
    if 1 == len(linestyle_list) < ndatas:
        # Set the same linestyles for all curves
        linestyle_list = list(repeat(linestyle_list[0], ndatas))
    if 1 == len(legend_list) < ndatas:
        # Set no legend for all curves
        legend_list = list(repeat("", ndatas))
        no_legend = True
    else:
        no_legend = False

    # Plot
    if type_plot == "curve":
        for i in range(ndatas):
            ax.plot(
                Xdatas[i_Xdatas[i]],
                Ydatas[i],
                color=color_list[i],
                label=legend_list[i],
                linewidth=linewidth_list[i],
                ls=linestyle_list[i],
            )
        if xticks is not None:
            ax.xaxis.set_ticks(xticks)
    elif type_plot == "bargraph":
        positions = range(-ndatas + 1, ndatas, 2)
        for i in range(ndatas):
            # width = (Xdatas[i_Xdatas[i]][1] - Xdatas[i_Xdatas[i]][0]) / ndatas
            width = Xdatas[i_Xdatas[i]][-1] / barwidth
            barlist = ax.bar(
                Xdatas[i_Xdatas[i]] + positions[i] * width / (2 * ndatas),
                Ydatas[i],
                color=color_list[i],
                width=width,
                label=legend_list[i],
            )
            if fund_harm is not None:  # Find fundamental
                imax = argmin(abs(Xdatas[i] - fund_harm))
                barlist[imax].set_edgecolor("k")
                barlist[imax].set_facecolor("k")

        if xticks is not None:
            ax.xaxis.set_ticks(xticks)
    elif type_plot == "barchart":
        for i in range(ndatas):
            if i == 0:
                ax.bar(
                    range(len(Xdatas[i_Xdatas[i]])),
                    Ydatas[i],
                    color=color_list[i],
                    width=0.5,
                    label=legend_list[i],
                )
            else:
                ax.bar(
                    range(len(Xdatas[i_Xdatas[i]])),
                    Ydatas[i],
                    edgecolor=color_list[i],
                    width=0.5,
                    fc="None",
                    lw=1,
                    label=legend_list[i],
                )
        plt.xticks(
            range(len(Xdatas[i_Xdatas[i]])),
            [str(f) for f in Xdatas[i_Xdatas[i]]],
            rotation=90,
        )
    elif type_plot == "quiver":
        for i in range(ndatas):
            x = [e[0] for e in Xdatas[i_Xdatas[i]]]
            y = [e[1] for e in Xdatas[i_Xdatas[i]]]
            vect_list = split(Ydatas[i], 2)
            ax.quiver(x, y, squeeze(vect_list[0]), squeeze(vect_list[1]))
            ax.axis("equal")

    ax.set_xlabel(xlabel)
    ax.set_ylabel(ylabel)
    ax.set_xlim([x_min, x_max])
    ax.set_ylim([y_min, y_max])

    if is_logscale_x:
        ax.set_xscale("log")

    if is_logscale_y:
        ax.set_yscale("log")

    if is_disp_title and title not in [None, ""]:
        ax.set_title(title)

    ax.grid(is_grid)

    if ndatas > 1 and not no_legend:
        ax.legend(prop={"family": FONT_NAME, "size": FONT_SIZE_LEGEND})

    plt.tight_layout()
    for item in (
        [ax.xaxis.label, ax.yaxis.label] + ax.get_xticklabels() + ax.get_yticklabels()
    ):
        item.set_fontname(FONT_NAME)
        item.set_fontsize(FONT_SIZE_LABEL)
    ax.title.set_fontname(FONT_NAME)
    ax.title.set_fontsize(FONT_SIZE_TITLE)

    if save_path is not None:
        fig.savefig(save_path)
        plt.close()

    if is_show_fig:
        fig.show()

    if win_title:
        fig.canvas.set_window_title(win_title)



class PointArc1Error(Exception):
    """ """

    pass


class RadiusArc1Error(Exception):
    """ """

    pass


class NbPointArc1DError(Exception):
    """ """

    pass


class AngleRotationArc1Error(Exception):
    """ """

    pass


class PointTranslateArc1Error(Exception):
    """ """

    pass



# -*- coding: utf-8 -*-
import os


def short_filepath(filepath, length=40):
    """
    truncate a filepath string to the specified length
    but at least returns the filename
    """
    filepath = filepath.replace("/", "\\")
    if len(filepath) < length:
        return filepath
    else:
        filename = os.path.split(filepath)[1]
        path = os.path.split(filepath)[0]
        length = length - len(filename) - 3
        path = "" if length < 0 else str(path)[-length:] + "\\"
        path = path.split("\\", 1)[-1]
        filepath = "..\\" + path + filename
    return filepath



# -*- coding: utf-8 -*-

from ....Classes.Lamination import Lamination
from ....Methods.Machine.LamSlotWind import *


def check(self):
    """Check that the Lamination object is correct

    Parameters
    ----------
    self :
        A LamSlotWind object

    Returns
    -------
    None

    Raises
    _______
    LWC_SlotTooLong
        The Slot is too long for the lamination (HYoke <0)
    LWC_MismatchPhase
        The Winding and the Converter don't have the same number of phase
    LWC_OverlappingSlot
        The Lamination has overlapping slot
    """

    Lamination.check(self)

    self.winding.conductor.check()
    self.slot.check()

    if self.comp_height_yoke() < 0:
        raise LWC_SlotTooLong("The Slot is too long for the lamination " "(HYoke <0)")

    """
    Wt = self.slot.comp_tooth_widths()["WTooth_min"]

    if Wt < 0:
        raise LWC_OverlappingSlot("The Lamination has overlapping slot")
    """



from ....Methods.Slot.Slot import SlotCheckError


class S61_InnerCheckError(SlotCheckError):
    """ """

    pass


class S61_W21CheckError(SlotCheckError):
    """ """

    pass


class S61_RCheckError(SlotCheckError):
    """ """

    pass


class S61_WindHError(SlotCheckError):
    """ """

    pass


class S61_WindWError(SlotCheckError):
    """ """

    pass


class S61_WindError(SlotCheckError):
    """

    Parameters
    ----------

    Returns
    -------

    Raises
    ------
    winding


    """

    pass



# -*- coding: utf-8 -*-

from PySide2.QtWidgets import QWidget, QTableWidgetItem
from ......GUI.Dialog.DMachineSetup.SPreview.WMachineTable.Ui_WMachineTable import (
    Ui_WMachineTable,
)
import matplotlib.pyplot as plt


class WMachineTable(Ui_WMachineTable, QWidget):
    """Table to display the main paramaters of the machine"""

    def __init__(self, parent=None):
        """Initialize the GUI

        Parameters
        ----------
        self : SWindCond
            A SWindCond widget
        """

        # Build the interface according to the .ui file
        QWidget.__init__(self, parent)
        self.setupUi(self)

        self.machine = None

        # Connect the widget
        self.b_mmf.clicked.connect(self.plot_mmf)
        self.b_plot_machine.clicked.connect(self.plot_machine)

    def update_tab(self, machine):
        """Update the table to match the machine

        Parameters
        ----------
        self : WMachineTable
            A WMachineTable object
        """

        self.machine = machine
        desc_dict = self.machine.comp_desc_dict()

        self.tab_param.clear()
        # Set header
        self.tab_param.setColumnCount(2)
        item = QTableWidgetItem("Name")
        self.tab_param.setHorizontalHeaderItem(0, item)
        item = QTableWidgetItem("Value")
        self.tab_param.setHorizontalHeaderItem(1, item)
        # Set containt
        for ii, desc in enumerate(desc_dict):
            if desc["value"] is not None:
                self.tab_param.insertRow(ii)
                self.tab_param.setItem(ii, 0, QTableWidgetItem(desc["verbose"]))
                if desc["type"] is float:
                    txt = format(desc["value"], ".4g")
                else:
                    txt = str(desc["value"])
                if desc["unit"] not in ["", None]:
                    txt += " " + desc["unit"]
                self.tab_param.setItem(ii, 1, QTableWidgetItem(txt))

    def plot_mmf(self):
        """Plot the unit mmf of the stator"""
        if self.machine is not None:
            self.machine.stator.plot_mmf_unit(is_show_fig=True)

    def plot_machine(self):
        """Plot the machine"""
        if self.machine is not None:
            self.machine.plot()



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Optimization/OptiGenAlgNsga2Deap.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Optimization/OptiGenAlgNsga2Deap
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .OptiGenAlg import OptiGenAlg

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Optimization.OptiGenAlgNsga2Deap.solve import solve
except ImportError as error:
    solve = error

try:
    from ..Methods.Optimization.OptiGenAlgNsga2Deap.mutate import mutate
except ImportError as error:
    mutate = error

try:
    from ..Methods.Optimization.OptiGenAlgNsga2Deap.cross import cross
except ImportError as error:
    cross = error

try:
    from ..Methods.Optimization.OptiGenAlgNsga2Deap.create_toolbox import create_toolbox
except ImportError as error:
    create_toolbox = error

try:
    from ..Methods.Optimization.OptiGenAlgNsga2Deap.check_optimization_input import (
        check_optimization_input,
    )
except ImportError as error:
    check_optimization_input = error

try:
    from ..Methods.Optimization.OptiGenAlgNsga2Deap.delete_toolbox import delete_toolbox
except ImportError as error:
    delete_toolbox = error


from ntpath import basename
from os.path import isfile
from ._check import CheckTypeError
import numpy as np
import random
from cloudpickle import dumps, loads
from ._check import CheckTypeError

try:
    from deap.base import Toolbox
except ImportError:
    Toolbox = ImportError
from ._check import InitUnKnowClassError
from .OptiProblem import OptiProblem
from .XOutput import XOutput


class OptiGenAlgNsga2Deap(OptiGenAlg):
    """Multi-objectives optimization problem with some constraints"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Optimization.OptiGenAlgNsga2Deap.solve
    if isinstance(solve, ImportError):
        solve = property(
            fget=lambda x: raise_(
                ImportError("Can't use OptiGenAlgNsga2Deap method solve: " + str(solve))
            )
        )
    else:
        solve = solve
    # cf Methods.Optimization.OptiGenAlgNsga2Deap.mutate
    if isinstance(mutate, ImportError):
        mutate = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use OptiGenAlgNsga2Deap method mutate: " + str(mutate)
                )
            )
        )
    else:
        mutate = mutate
    # cf Methods.Optimization.OptiGenAlgNsga2Deap.cross
    if isinstance(cross, ImportError):
        cross = property(
            fget=lambda x: raise_(
                ImportError("Can't use OptiGenAlgNsga2Deap method cross: " + str(cross))
            )
        )
    else:
        cross = cross
    # cf Methods.Optimization.OptiGenAlgNsga2Deap.create_toolbox
    if isinstance(create_toolbox, ImportError):
        create_toolbox = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use OptiGenAlgNsga2Deap method create_toolbox: "
                    + str(create_toolbox)
                )
            )
        )
    else:
        create_toolbox = create_toolbox
    # cf Methods.Optimization.OptiGenAlgNsga2Deap.check_optimization_input
    if isinstance(check_optimization_input, ImportError):
        check_optimization_input = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use OptiGenAlgNsga2Deap method check_optimization_input: "
                    + str(check_optimization_input)
                )
            )
        )
    else:
        check_optimization_input = check_optimization_input
    # cf Methods.Optimization.OptiGenAlgNsga2Deap.delete_toolbox
    if isinstance(delete_toolbox, ImportError):
        delete_toolbox = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use OptiGenAlgNsga2Deap method delete_toolbox: "
                    + str(delete_toolbox)
                )
            )
        )
    else:
        delete_toolbox = delete_toolbox
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        toolbox=None,
        selector=None,
        crossover=None,
        mutator=None,
        p_cross=0.9,
        p_mutate=0.1,
        size_pop=40,
        nb_gen=100,
        problem=-1,
        xoutput=-1,
        logger_name="Pyleecan.OptiSolver",
        is_keep_all_output=False,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "toolbox" in list(init_dict.keys()):
                toolbox = init_dict["toolbox"]
            if "selector" in list(init_dict.keys()):
                selector = init_dict["selector"]
            if "crossover" in list(init_dict.keys()):
                crossover = init_dict["crossover"]
            if "mutator" in list(init_dict.keys()):
                mutator = init_dict["mutator"]
            if "p_cross" in list(init_dict.keys()):
                p_cross = init_dict["p_cross"]
            if "p_mutate" in list(init_dict.keys()):
                p_mutate = init_dict["p_mutate"]
            if "size_pop" in list(init_dict.keys()):
                size_pop = init_dict["size_pop"]
            if "nb_gen" in list(init_dict.keys()):
                nb_gen = init_dict["nb_gen"]
            if "problem" in list(init_dict.keys()):
                problem = init_dict["problem"]
            if "xoutput" in list(init_dict.keys()):
                xoutput = init_dict["xoutput"]
            if "logger_name" in list(init_dict.keys()):
                logger_name = init_dict["logger_name"]
            if "is_keep_all_output" in list(init_dict.keys()):
                is_keep_all_output = init_dict["is_keep_all_output"]
        # Set the properties (value check and convertion are done in setter)
        self.toolbox = toolbox
        # Call OptiGenAlg init
        super(OptiGenAlgNsga2Deap, self).__init__(
            selector=selector,
            crossover=crossover,
            mutator=mutator,
            p_cross=p_cross,
            p_mutate=p_mutate,
            size_pop=size_pop,
            nb_gen=nb_gen,
            problem=problem,
            xoutput=xoutput,
            logger_name=logger_name,
            is_keep_all_output=is_keep_all_output,
        )
        # The class is frozen (in OptiGenAlg init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        OptiGenAlgNsga2Deap_str = ""
        # Get the properties inherited from OptiGenAlg
        OptiGenAlgNsga2Deap_str += super(OptiGenAlgNsga2Deap, self).__str__()
        OptiGenAlgNsga2Deap_str += "toolbox = " + str(self.toolbox) + linesep + linesep
        return OptiGenAlgNsga2Deap_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from OptiGenAlg
        if not super(OptiGenAlgNsga2Deap, self).__eq__(other):
            return False
        if other.toolbox != self.toolbox:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from OptiGenAlg
        diff_list.extend(super(OptiGenAlgNsga2Deap, self).compare(other, name=name))
        if (other.toolbox is None and self.toolbox is not None) or (
            other.toolbox is not None and self.toolbox is None
        ):
            diff_list.append(name + ".toolbox None mismatch")
        elif self.toolbox is not None:
            diff_list.extend(
                self.toolbox.compare(other.toolbox, name=name + ".toolbox")
            )
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from OptiGenAlg
        S += super(OptiGenAlgNsga2Deap, self).__sizeof__()
        S += getsizeof(self.toolbox)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from OptiGenAlg
        OptiGenAlgNsga2Deap_dict = super(OptiGenAlgNsga2Deap, self).as_dict(**kwargs)
        if self.toolbox is None:
            OptiGenAlgNsga2Deap_dict["toolbox"] = None
        else:
            # Store serialized data (using cloudpickle) and str
            # to read it in json save files
            OptiGenAlgNsga2Deap_dict["toolbox"] = {
                "__class__": str(type(self._toolbox)),
                "__repr__": str(self._toolbox.__repr__()),
                "serialized": dumps(self._toolbox).decode("ISO-8859-2"),
            }
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        OptiGenAlgNsga2Deap_dict["__class__"] = "OptiGenAlgNsga2Deap"
        return OptiGenAlgNsga2Deap_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.toolbox = None
        # Set to None the properties inherited from OptiGenAlg
        super(OptiGenAlgNsga2Deap, self)._set_None()

    def _get_toolbox(self):
        """getter of toolbox"""
        return self._toolbox

    def _set_toolbox(self, value):
        """setter of toolbox"""
        check_var("toolbox", value, "Toolbox")
        self._toolbox = value

    toolbox = property(
        fget=_get_toolbox,
        fset=_set_toolbox,
        doc=u"""DEAP toolbox

        :Type: deap.base.Toolbox
        """,
    )



# -*- coding: utf-8 -*-


def comp_mass_magnets(self):
    """Compute the mass of the hole magnets (some of them may be missing)

    Parameters
    ----------
    self : HoleMag
        A HoleMag object

    Returns
    -------
    Mmag: float
        mass of the magnets [kg]
    """

    M = 0
    mag_list = self.get_magnet_list()
    for ii, mag in enumerate(mag_list):
        if mag is not None:
            Smag = self.comp_surface_magnet_id(ii)
            M += Smag * mag.Lmag * mag.mat_type.struct.rho

    return M



# -*-- coding: utf-8 -*
def discretize(self, nb_point):
    """Returns the discretize version of the PolarArc

    Parameters
    ----------
    nb_point : int
        number of points wanted per line

    Returns
    -------
    point_list : list
        List of complex coordinate of the points

    """
    # check if the PolarArc is correct
    self.check()

    if not isinstance(nb_point, int):
        raise NbPointPolarArcError("Discretize : the nb_point must be an integer")
    if nb_point < 0:
        raise NbPointPolarArcError("Discretize: nb_point must be >= 0")

    # getting lines that delimit the PolarArc
    lines = self.get_lines()
    point_list = list()
    for line in lines:
        point_list.extend(line.discretize(nb_point))
        # The begin of the next line is the end of the current one
        point_list.pop()
    return point_list


class NbPointPolarArcError(Exception):
    """ """

    pass



# -*- coding: utf-8 -*-

import numpy as np


def make_ndarray_equal(vec_ref, vec_to_sort):
    """Compare two ndarray, sort the second to match with the first one.

    Parameters
    ----------
    vec_ref : ndarray
        reference vector
    vec_to_sort : ndarray
        vector to sort

    Returns
    -------
    Isort: ndarray
        Vector of indice to sort the vec_to_sort
    vec: ndarray
        Sorted vector

    """

    nb = len(vec_ref)
    Isort = np.zeros(nb)

    for i in range(nb):
        Isort[i] = np.where(vec_ref[i] == vec_to_sort)

    return Isort, vec_to_sort[Isort]



# -*- coding: utf-8 -*-
from numpy import array


def comp_height_opening(self, Ndisc=200):
    """Compute the height of the opening area (Hslot - Hactive)

    Parameters
    ----------
    self : Slot
        A Slot object
    Ndisc : int
        Number of point to discretize the lines

    Returns
    -------
    Hwind: float
        Height of the opening area [m]

    """

    return self.comp_height(Ndisc=Ndisc) - self.comp_height_active(Ndisc=Ndisc)



# -*- coding: utf-8 -*-

from SciDataTool import Data1D, DataTime, DataFreq


def get_data(self):
    """Generate Data objects

    Parameters
    ----------
    self : ImportData
        An ImportData object

    Returns
    -------
    Data: DataND
        The generated Data object

    """

    axes_list = []
    is_freq = False
    for axis in self.axes:
        if axis.name == "freqs" or axis.name == "wavenumber":
            is_freq = True
        axes_list.append(
            Data1D(
                values=axis.field.get_data(),
                name=axis.name,
                unit=axis.unit,
                symmetries=axis.symmetries,
                normalizations=axis.normalizations,
            )
        )

    if is_freq:
        Data = DataFreq(
            axes=axes_list,
            values=self.field.get_data(),
            name=self.name,
            symbol=self.symbol,
            unit=self.unit,
            normalizations=self.normalizations,
        )
    else:
        Data = DataTime(
            axes=axes_list,
            values=self.field.get_data(),
            name=self.name,
            symbol=self.symbol,
            unit=self.unit,
            normalizations=self.normalizations,
        )

    return Data



from numpy import argmax, cos, abs as np_abs, angle as np_angle


def comp_angle_d_axis(self):
    """Compute the angle between the X axis and the first d+ axis
    By convention a "Tooth" is centered on the X axis

    Parameters
    ----------
    self : LamSlotWind
        A LamSlotWind object

    Returns
    -------
    d_angle : float
        angle between the X axis and the first d+ axis
    """

    if self.winding is None or self.winding.qs == 0:
        return 0

    p = self.get_pole_pair_number()

    MMF, _ = self.comp_mmf_unit(Nt=1, Na=400 * p)

    # Get angle values
    results1 = MMF.get_along("angle[oneperiod]")
    angle_stator = results1["angle"]

    # Get the unit mmf FFT and wavenumbers
    results = MMF.get_along("wavenumber")
    wavenumber = results["wavenumber"]
    mmf_ft = results[MMF.symbol]

    # Find the fundamental harmonic of MMF
    indr_fund = np_abs(wavenumber - p).argmin()
    phimax = np_angle(mmf_ft[indr_fund])
    magmax = np_abs(mmf_ft[indr_fund])

    # Reconstruct fundamental MMF wave
    mmf_waveform = magmax * cos(p * angle_stator + phimax)

    # Get the first angle where mmf is max
    d_angle = angle_stator[argmax(mmf_waveform)]

    # import matplotlib.pyplot as plt
    # from numpy import squeeze

    # fig, ax = plt.subplots()
    # ax.plot(angle_stator, squeeze(MMF.get_along("angle[oneperiod]")[MMF.symbol]), "k")
    # ax.plot(angle_stator, mmf_waveform, "r")
    # ax.plot([d_angle, d_angle], [-magmax, magmax], "--k")
    # plt.show()

    return d_angle



# -*- coding: utf-8 -*-


def get_pole_pair_number(self):
    """Return the number of pair of pole of the Lamination

    Parameters
    ----------
    self : LamSlotWind
        A LamSlotWind object

    Returns
    -------
    p: int
        Number of pair of pole

    """

    if self.winding is not None:
        return self.winding.p
    else:
        return None
